@@ -7,8 +7,8 @@ license: GPL
 """
 #============================================================
 # $Source: /home/ncq/Projekte/cvs2git/vcs-mirror/gnumed/gnumed/client/business/gmDemographicRecord.py,v $
-# $Id: gmDemographicRecord.py,v 1.103 2009-11-18 16:10:02 ncq Exp $
-__version__ = "$Revision: 1.103 $"
+# $Id: gmDemographicRecord.py,v 1.106 2010-01-31 18:12:53 ncq Exp $
+__version__ = "$Revision: 1.106 $"
 __author__ = "K.Hilbert <Karsten.Hilbert@gmx.net>, I.Haywood <ihaywood@gnu.org>"
 
 # stdlib
@@ -23,12 +23,29 @@ import mx.DateTime as mxDT
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
 from Gnumed.pycommon import gmDispatcher, gmBusinessDBObject, gmPG2, gmTools
-from Gnumed.business import gmMedDoc
 
 
 _log = logging.getLogger('gm.business')
 _log.info(__version__)
 
+#============================================================
+def get_countries():
+	cmd = u"""
+		select
+			_(name) as l10n_country, name, code, deprecated
+		from dem.country
+		order by l10n_country"""
+	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd}])
+	return rows
+#============================================================
+def get_country_for_region(region=None):
+	cmd = u"""
+SELECT code_country, l10n_country FROM dem.v_state WHERE l10n_state = %(region)s
+	union
+SELECT code_country, l10n_country FROM dem.v_state WHERE state = %(region)s
+"""
+	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': {'region': region}}])
+	return rows
 #============================================================
 def delete_province(province=None, delete_urbs=False):
 
@@ -551,6 +568,9 @@ def _post_patient_selection(**kwargs):
 #------------------------------------------------------------
 if __name__ == "__main__":
 
+	if len(sys.argv) < 2:
+		sys.exit()
+
 	import random
 	#--------------------------------------------------------
 	def test_address_exists():
@@ -600,15 +620,23 @@ if __name__ == "__main__":
 		print address
 		print "deleted address:", delete_address(address)
 	#--------------------------------------------------------
+	def test_get_countries():
+		for c in get_countries():
+			print c
+	#--------------------------------------------------------
+	def test_get_country_for_region():
+		region = raw_input("Please enter a region: ")
+		print "country for region [%s] is: %s" % (region, get_country_for_region(region = region))
+	#--------------------------------------------------------
+	if sys.argv[1] != 'test':
+		sys.exit()
 
-	try:
-		gmPG2.get_connection()
+	#gmPG2.get_connection()
 
-		test_address_exists()
-		test_create_address()
-	except:
-		_log.exception('test suite failed')
-		raise
+	#test_address_exists()
+	#test_create_address()
+	#test_get_countries()
+	test_get_country_for_region()
 
 	sys.exit()
 
@@ -635,7 +663,16 @@ if __name__ == "__main__":
 		print "--------------------------------------"
 #============================================================
 # $Log: gmDemographicRecord.py,v $
-# Revision 1.103  2009-11-18 16:10:02  ncq
+# Revision 1.106  2010-01-31 18:12:53  ncq
+# - get-country-for-region
+#
+# Revision 1.105  2010/01/31 16:51:21  ncq
+# - fix get-countries()
+#
+# Revision 1.104  2010/01/31 16:32:19  ncq
+# - get-countries()
+#
+# Revision 1.103  2009/11/18 16:10:02  ncq
 # - more provinces handling
 #
 # Revision 1.102  2009/11/17 19:40:40  ncq
