@@ -4,7 +4,7 @@
 license: GPL
 """
 #============================================================
-__version__ = "$Revision: 1.157.2.1 $"
+__version__ = "$Revision: 1.157 $"
 __author__ = "Carlos Moro <cfmoro1976@yahoo.es>, <karsten.hilbert@gmx.net>"
 
 import types, sys, string, datetime, logging, time
@@ -188,27 +188,12 @@ class cHealthIssue(gmBusinessDBObject.cBusinessDBObject):
 			return u'<???>'
 
 		return gmDateTime.format_interval_medically(self._payload[self._idx['age_noted']])
-
-#		# seemingly silly but convinces PG to "nicely"
-#		# format the interval for us
-#		cmd = u"""select
-#age (
-#	(select dob from dem.identity where pk = %(pat)s) + %(issue_age)s,
-#	(select dob from dem.identity where pk = %(pat)s)
-#)::text
-#|| ' (' || age (
-#	(select dob from dem.identity where pk = %(pat)s) + %(issue_age)s
-#)::text || ' ago)'
-#"""
-#		args = {
-#			'pat': self._payload[self._idx['pk_patient']],
-#			'issue_age': self._payload[self._idx['age_noted']]
-#		}
-#		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}])
-#		return rows[0][0]
 	#--------------------------------------------------------
 	def _get_laterality_description(self):
-		return laterality2str[self._payload[self._idx['laterality']]]
+		try:
+			return laterality2str[self._payload[self._idx['laterality']]]
+		except KeyError:
+			return u'??'
 
 	laterality_description = property(_get_laterality_description, lambda x:x)
 	#--------------------------------------------------------
@@ -234,7 +219,7 @@ class cHealthIssue(gmBusinessDBObject.cBusinessDBObject):
 			self._payload[self._idx['description']],
 			u'\u00AB',
 			gmTools.coalesce (
-				initial = laterality2str[self._payload[self._idx['laterality']]],
+				initial = self.laterality_description,		#laterality2str[self._payload[self._idx['laterality']]],
 				instead = u'',
 				template_initial = u' (%s)',
 				none_equivalents = [None, u'', u'?']
@@ -357,21 +342,22 @@ class cHealthIssue(gmBusinessDBObject.cBusinessDBObject):
 		del procs
 
 		epis = self.get_episodes()
-		# documents
-		doc_folder = patient.get_document_folder()
-		docs = doc_folder.get_documents(episodes = [ e['pk_episode'] for e in epis ])
+		if len(epis) > 0:
+			# documents
+			doc_folder = patient.get_document_folder()
+			docs = doc_folder.get_documents(episodes = [ e['pk_episode'] for e in epis ])
 
-		if len(docs) > 0:
-			lines.append(u'')
-			lines.append(_('Documents: %s') % len(docs))
-		del docs
+			if len(docs) > 0:
+				lines.append(u'')
+				lines.append(_('Documents: %s') % len(docs))
+			del docs
 
-		# rest results
-		tests = emr.get_test_results_by_date(episodes = [ e['pk_episode'] for e in epis ])
-		if len(tests) > 0:
-			lines.append(u'')
-			lines.append(_('Measurements and Results: %s') % len(tests))
-		del tests
+			# test results
+			tests = emr.get_test_results_by_date(episodes = [ e['pk_episode'] for e in epis ])
+			if len(tests) > 0:
+				lines.append(u'')
+				lines.append(_('Measurements and Results: %s') % len(tests))
+			del tests
 
 		del epis
 
@@ -813,17 +799,15 @@ class cEncounter(gmBusinessDBObject.cBusinessDBObject):
 		'assessment_of_encounter'
 	]
 	#--------------------------------------------------------
-	def set_active(self, staff_id=None):
+	def set_active(self):
 		"""Set the enconter as the active one.
 
 		"Setting active" means making sure the encounter
 		row has the youngest "last_affirmed" timestamp of
 		all encounter rows for this patient.
-
-		staff_id - Provider's primary key
 		"""
-		self._payload[self._idx['last_affirmed']] = datetime.datetime.now(tz = gmDateTime.gmCurrentLocalTimezone)
-		self.save_payload()
+		self['last_affirmed'] = gmDateTime.pydt_now_here()
+		self.save()
 	#--------------------------------------------------------
 	def transfer_clinical_data(self, source_episode=None, target_episode=None):
 		"""
@@ -862,11 +846,13 @@ class cEncounter(gmBusinessDBObject.cBusinessDBObject):
 		relevant_fields = [
 			'pk_location',
 			'pk_type',
+			'pk_patient',
 			'reason_for_encounter',
 			'assessment_of_encounter'
 		]
 		for field in relevant_fields:
 			if self._payload[self._idx[field]] != another_object[field]:
+				_log.debug('mismatch on [%s]: "%s" vs. "%s"', field, self._payload[self._idx[field]], another_object[field])
 				return False
 
 		relevant_fields = [
@@ -877,12 +863,15 @@ class cEncounter(gmBusinessDBObject.cBusinessDBObject):
 			if self._payload[self._idx[field]] is None:
 				if another_object[field] is None:
 					continue
+				_log.debug('mismatch on [%s]: "%s" vs. "%s"', field, self._payload[self._idx[field]], another_object[field])
 				return False
 
 			if another_object[field] is None:
 				return False
 
-			if self._payload[self._idx[field]].strftime('%Y-%m-%d %H:%M:%S %Z') != another_object[field].strftime('%Y-%m-%d %H:%M:%S %Z'):
+			#if self._payload[self._idx[field]].strftime('%Y-%m-%d %H:%M:%S %Z') != another_object[field].strftime('%Y-%m-%d %H:%M:%S %Z'):
+			if self._payload[self._idx[field]].strftime('%Y-%m-%d %H:%M') != another_object[field].strftime('%Y-%m-%d %H:%M'):
+				_log.debug('mismatch on [%s]: "%s" vs. "%s"', field, self._payload[self._idx[field]], another_object[field])
 				return False
 
 		return True
@@ -1495,6 +1484,12 @@ def delete_performed_procedure(procedure=None):
 #------------------------------------------------------------
 if __name__ == '__main__':
 
+	if len(sys.argv) < 2:
+		sys.exit()
+
+	if sys.argv[1] != 'test':
+		sys.exit()
+
 	#--------------------------------------------------------
 	# define tests
 	#--------------------------------------------------------
@@ -1592,565 +1587,13 @@ if __name__ == '__main__':
 			print type(diagnostic_certainty_classification2str(t)), diagnostic_certainty_classification2str(t)
 
 	#--------------------------------------------------------
-	if (len(sys.argv) > 1) and (sys.argv[1] == 'test'):
-		# run them
-		#test_episode()
-		#test_problem()
-		#test_encounter()
-		#test_health_issue()
-		#test_hospital_stay()
-		#test_performed_procedure()
-		test_diagnostic_certainty_classification_map()
+	# run them
+	#test_episode()
+	#test_problem()
+	#test_encounter()
+	#test_health_issue()
+	#test_hospital_stay()
+	#test_performed_procedure()
+	test_diagnostic_certainty_classification_map()
 #============================================================
-# $Log: gmEMRStructItems.py,v $
-# Revision 1.157.2.1  2010-02-06 19:11:59  ncq
-# - fix _()
-#
-# Revision 1.157  2010/01/21 08:40:15  ncq
-# - make diagnostic certainty conversion robust against when _() is imported
-# - enable problem to return certainty description
-# - test certainty mapper
-#
-# Revision 1.156  2009/12/21 14:58:19  ncq
-# - for issue/episode show creation encounter
-# - %x -> %Y-%m-%d timestamp formatting
-#
-# Revision 1.155  2009/11/28 18:35:23  ncq
-# - fix health_issue2problem/episode2problem
-# - add is_dummy to dummy health issue
-# - enhance cProblem to allow for potential problems, too
-#
-# Revision 1.154  2009/11/13 21:01:45  ncq
-# - create-performed-proc
-#
-# Revision 1.153  2009/11/06 15:03:15  ncq
-# - better known-since formatting for health issue
-# - include meds in class formatting
-#
-# Revision 1.152  2009/10/20 10:24:03  ncq
-# - laterality/certainty properties
-#
-# Revision 1.151  2009/09/29 13:14:06  ncq
-# - fix faulty param in create_health_issue
-#
-# Revision 1.150  2009/09/23 14:31:25  ncq
-# - better ABCD long desc
-# - health issue
-# 	- teach init/create-* to take advantage of patient pk
-# - better formatting
-# - cProcedure etc
-#
-# Revision 1.149  2009/09/17 21:51:27  ncq
-# - add performed procedures support
-#
-# Revision 1.148  2009/09/15 15:24:48  ncq
-# - add format() to hospital stays and use it
-#
-# Revision 1.147  2009/09/13 18:22:55  ncq
-# - ignore return of save_payload() - it's a dummy
-#
-# Revision 1.146  2009/09/01 23:03:57  ncq
-# - better classification
-#
-# Revision 1.145  2009/09/01 22:14:15  ncq
-# - support diagnostic certainty on issues and episodes
-#
-# Revision 1.144  2009/07/16 09:51:53  ncq
-# - properly update enc type and check success
-#
-# Revision 1.143  2009/07/06 14:56:04  ncq
-# - consolidate date formatting
-# - use improved test results formatting
-#
-# Revision 1.142  2009/07/02 20:46:17  ncq
-# - get-episodes in issue
-# - include docs/tests in issue/episode formatting
-#
-# Revision 1.141  2009/07/01 17:05:56  ncq
-# - cleanup
-#
-# Revision 1.140  2009/06/29 14:59:18  ncq
-# - add get-latest-soap
-#
-# Revision 1.139  2009/06/20 12:33:52  ncq
-# - improved episode formatting as per list
-#
-# Revision 1.138  2009/06/04 14:32:16  ncq
-# - reimport lost comment
-#
-# Revision 1.138  2009/05/28 10:45:33  ncq
-# - comment added
-#
-# Revision 1.137  2009/04/13 10:52:14  ncq
-# - support same_payload on encounter
-#
-# Revision 1.136  2009/04/05 17:48:52  ncq
-# - support grouping
-#
-# Revision 1.135  2009/04/03 11:07:25  ncq
-# - include stays in issue formatting
-#
-# Revision 1.134  2009/04/03 10:39:40  ncq
-# - include hospital stays into episode formatting
-#
-# Revision 1.133  2009/04/03 09:31:13  ncq
-# - add hospital stay API
-#
-# Revision 1.132  2009/02/27 12:38:03  ncq
-# - improved SOAP formatting
-#
-# Revision 1.131  2009/02/23 08:46:01  ncq
-# - fix faulty column label
-#
-# Revision 1.130  2009/02/17 17:45:53  ncq
-# - fix create_health_issue
-#
-# Revision 1.129  2008/12/18 21:26:20  ncq
-# - u''ify some strings
-#
-# Revision 1.128  2008/12/09 23:21:00  ncq
-# - .date -> .clin_when in documents
-# - no more fk_patient in episode
-#
-# Revision 1.127  2008/12/01 12:36:13  ncq
-# - much improved formatting
-#
-# Revision 1.126  2008/11/24 11:09:01  ncq
-# - health issues now stem from a view
-# - no more fk_patient in clin.health_issue
-# - no more patient id in create_episode
-#
-# Revision 1.125  2008/11/20 18:40:53  ncq
-# - health_issue/episode2problem
-# - improved formatting
-#
-# Revision 1.124  2008/10/22 12:04:55  ncq
-# - use %x in strftime
-#
-# Revision 1.123  2008/10/12 15:13:30  ncq
-# - no more "foundational" in health issue
-#
-# Revision 1.122  2008/09/09 19:55:07  ncq
-# - Jerzy found a misspelling
-#
-# Revision 1.121  2008/09/04 11:52:17  ncq
-# - append an empty line per soap category
-#
-# Revision 1.120  2008/09/02 18:58:27  ncq
-# - fk_patient dropped from clin.health_issue
-#
-# Revision 1.119  2008/08/17 18:13:39  ncq
-# - add CRLF after date/time/provider in soap formatting as
-#   suggested by Rogerio on the list
-#
-# Revision 1.118  2008/07/24 13:57:51  ncq
-# - update/create/delete_encounter_type
-#
-# Revision 1.117  2008/07/22 13:53:12  ncq
-# - cleanup
-#
-# Revision 1.116  2008/07/14 13:44:38  ncq
-# - add .format to episode
-# - factor out .format_soap from .format on encounter
-# - better visualize soap sections in output as per user request
-#
-# Revision 1.115  2008/07/12 15:20:30  ncq
-# - add format to health issue
-#
-# Revision 1.114  2008/06/26 21:17:59  ncq
-# - episode formatting: include docs
-# - encounter formatting: include results and docs
-#
-# Revision 1.113  2008/06/24 16:53:58  ncq
-# - include test results in encounter formatting such
-#   as to be included in the EMR tree browser :-)
-#
-# Revision 1.112  2008/05/19 15:43:45  ncq
-# - adapt to TZ code changes
-#
-# Revision 1.111  2008/05/13 14:06:17  ncq
-# - remove superfluous \n
-# - add missing .
-#
-# Revision 1.110  2008/04/11 12:20:52  ncq
-# - format() on episode and encounter
-#
-# Revision 1.109  2008/03/05 22:24:31  ncq
-# - support fk_encounter in issue and episode creation
-#
-# Revision 1.108  2008/02/25 17:29:59  ncq
-# - logging cleanup
-#
-# Revision 1.107  2008/01/30 13:34:49  ncq
-# - switch to std lib logging
-#
-# Revision 1.106  2008/01/22 11:49:14  ncq
-# - cleanup
-# - free-standing -> Unattributed as per list
-#
-# Revision 1.105  2008/01/16 19:36:17  ncq
-# - improve get_encounter_types()
-#
-# Revision 1.104  2008/01/13 01:12:53  ncq
-# - age_noted_human_readable()
-#
-# Revision 1.103  2007/10/29 11:04:11  ncq
-# - properly handle NULL pk_health_issue in create_apisode() thereby
-#   finding dupes in free-standing episodes, too
-# - this then asks for an explicit allow_dupes (defaulted to False)
-#   in create_episode() as we may, indeed, wish to allow dupes
-#   sometimes
-#
-# Revision 1.102  2007/10/11 12:00:17  ncq
-# - add has_narrative() and has_documents()
-#
-# Revision 1.101  2007/09/07 10:55:55  ncq
-# - get_dummy_health_issue()
-#
-# Revision 1.100  2007/08/15 14:56:30  ncq
-# - delete_health_issue()
-#
-# Revision 1.99  2007/05/18 13:25:56  ncq
-# - fix cEncounter.transfer_clinical_data()
-#
-# Revision 1.98  2007/05/14 10:32:50  ncq
-# - raise DatabaseObjectInUseError on psycopg2 integrity error
-#
-# Revision 1.97  2007/04/27 22:54:13  ncq
-# - when checking for existing episodes need to check
-#   associated health issue, too, of course
-#
-# Revision 1.96  2007/04/02 18:35:20  ncq
-# - create_encounter now more exception-y
-#
-# Revision 1.95  2007/03/18 13:01:55  ncq
-# - a bit of cleanup
-#
-# Revision 1.94  2007/01/09 18:01:32  ncq
-# - let exceptions report errors
-#
-# Revision 1.93  2007/01/09 12:56:02  ncq
-# - create_episode() now always takes patient fk
-#
-# Revision 1.92  2007/01/04 22:50:04  ncq
-# - allow changing fk_patient in cEpisode
-#
-# Revision 1.91  2007/01/02 16:14:41  ncq
-# - fix close_expired_episode()
-#
-# Revision 1.90  2006/12/25 22:48:52  ncq
-# - add cEncounter.has_clinical_data()
-#
-# Revision 1.89  2006/12/22 16:53:31  ncq
-# - use timezone definition in gmDateTime
-#
-# Revision 1.88  2006/11/24 09:30:33  ncq
-# - make cHealthIssue save more of its members
-# - if_patient -> fk_patient
-# - do not log i18n()ed message so failure there doesn't stop us from creating a health issue or episode
-#
-# Revision 1.87  2006/11/05 17:02:25  ncq
-# - enable health issue and episode to be inited from row
-#
-# Revision 1.86  2006/10/28 14:59:38  ncq
-# - __ -> _
-#
-# Revision 1.85  2006/10/28 14:59:20  ncq
-# - when reading from views no need to explicitely load xmin_*, it's part of the view anyways
-# - reduce query duplication by reuse of _cmd_fetch_payload
-#
-# Revision 1.84  2006/10/10 07:26:37  ncq
-# - no more clinitem exceptions
-#
-# Revision 1.83  2006/10/09 12:18:18  ncq
-# - convert to gmPG2
-# - convert to cBusinessDBObject
-# - unicode queries
-# - robustified test suite
-#
-# Revision 1.82  2006/09/03 11:27:24  ncq
-# - use gmNull.cNull
-# - add cHealthIssue.get_open_episode()
-# - add cEpisode.get_access_range()
-#
-# Revision 1.81  2006/07/19 20:25:00  ncq
-# - gmPyCompat.py is history
-#
-# Revision 1.80  2006/06/26 12:27:53  ncq
-# - cleanup
-# - add close_episode() and has_open_episode() to cHealthIssue
-#
-# Revision 1.79  2006/06/05 22:00:53  ncq
-# - must be "episode_open", not "is_open"
-#
-# Revision 1.78  2006/05/06 18:53:56  ncq
-# - select age(...) <> ...; -> select ... <> now() - ...; as per Syan
-#
-# Revision 1.77  2006/03/09 21:11:49  ncq
-# - spell out rfe/aoe
-#
-# Revision 1.76  2006/02/27 22:38:36  ncq
-# - spell out rfe/aoe as per Richard's request
-#
-# Revision 1.75  2005/12/26 05:26:38  sjtan
-#
-# match schema
-#
-# Revision 1.74  2005/12/25 13:24:30  sjtan
-#
-# schema changes in names .
-#
-# Revision 1.73  2005/12/06 17:57:13  ncq
-# - more id->pk fixes
-#
-# Revision 1.72  2005/12/06 14:24:14  ncq
-# - clin.clin_health_issue/episode -> clin.health_issue/episode
-#
-# Revision 1.71  2005/11/27 12:56:19  ncq
-# - add get_encounter_types()
-#
-# Revision 1.70  2005/11/27 12:44:57  ncq
-# - clinical tables are in schema "clin" now
-#
-# Revision 1.69  2005/10/15 18:15:37  ncq
-# - cleanup
-# - clin_encounter has fk_*, not pk_*
-# - remove clin_encounter.pk_provider support
-# - fix cEncounter.set_active()
-# - comment on that transfer_clinical_data will work but will not
-#   notify others about its changes
-#
-# Revision 1.68  2005/10/12 22:31:13  ncq
-# - encounter['rfe'] not mandatory anymore, hence don't need default
-# - encounters don't have a provider
-#
-# Revision 1.67  2005/10/11 21:49:36  ncq
-# - make create_encounter oblivious of emr object again
-#
-# Revision 1.66  2005/10/08 12:33:09  sjtan
-# tree can be updated now without refetching entire cache; done by passing emr object to create_xxxx methods and calling emr.update_cache(key,obj);refresh_historical_tree non-destructively checks for changes and removes removed nodes and adds them if cache mismatch.
-#
-# Revision 1.65  2005/10/04 19:21:31  sjtan
-# unicode and str are different but printable types.
-#
-# Revision 1.64  2005/09/25 01:00:47  ihaywood
-# bugfixes
-#
-# remember 2.6 uses "import wx" not "from wxPython import wx"
-# removed not null constraint on clin_encounter.rfe as has no value on instantiation
-# client doesn't try to set clin_encounter.description as it doesn't exist anymore
-#
-# Revision 1.63  2005/09/22 15:45:11  ncq
-# - clin_encounter.fk_provider removed
-#
-# Revision 1.62  2005/09/19 16:32:42  ncq
-# - add rfe/aoe support to clin_encounter
-# - remove get_aoes()/get_rfes()
-#
-# Revision 1.61  2005/09/12 15:05:58  ncq
-# - improve close_expired_episodes()
-#
-# Revision 1.60  2005/09/11 17:23:53  ncq
-# - close_expired_episodes()
-# - support is_open when adding episode
-#
-# Revision 1.59  2005/08/22 13:02:46  ncq
-# - prepare for 0.2
-#
-# Revision 1.58  2005/07/02 18:16:58  ncq
-# - default encounter type "in surgery" for 0.1, to become
-#   "in surgery"-on-write later on
-#
-# Revision 1.57  2005/06/23 14:58:51  ncq
-# - clean up transfer_clinical_data()
-#
-# Revision 1.56  2005/06/20 18:48:51  ncq
-# - a little cleanup in transfer_data
-#
-# Revision 1.55  2005/06/20 13:03:38  cfmoro
-# Relink encounter to another episode
-#
-# Revision 1.54  2005/06/15 22:25:29  ncq
-# - issue.rename()
-#
-# Revision 1.53  2005/06/14 18:53:37  ncq
-# - really do rename in rename(), needs to set _is_modified to work
-#
-# Revision 1.52  2005/06/12 21:40:42  ncq
-# - cleanup
-#
-# Revision 1.51  2005/05/14 15:05:40  ncq
-# - show HH:MM in auto-created encounters
-#
-# Revision 1.50  2005/04/25 08:28:28  ncq
-# - episode now has .description
-#
-# Revision 1.49  2005/04/24 14:42:22  ncq
-# - add age_noted as changable
-#
-# Revision 1.48  2005/04/03 20:05:38  ncq
-# - cEpisode.set_active() doesn't make sense no more
-#
-# Revision 1.47  2005/03/29 07:22:38  ncq
-# - improve text for auto generated encounters
-#
-# Revision 1.46  2005/03/23 18:31:19  ncq
-# - v_patient_items -> v_pat_items
-#
-# Revision 1.45  2005/03/20 16:47:26  ncq
-# - cleanup
-#
-# Revision 1.44  2005/03/17 21:59:35  cfmoro
-# Fixed log comment
-#
-# Revision 1.43  2005/03/17 21:46:23  cfmoro
-# Simplified cEpisode.rename function
-#
-# Revision 1.42  2005/03/17 21:14:45  cfmoro
-# Improved exception handling in get_as_episode.
-#
-# Revision 1.41  2005/03/17 13:35:52  ncq
-# - cleanup and streamlining
-#
-# Revision 1.40  2005/03/16 19:10:06  cfmoro
-# Added cProblem.get_as_episode method
-#
-# Revision 1.39  2005/03/14 14:28:54  ncq
-# - id_patient -> pk_patient
-# - properly handle simplified episode naming in create_episode()
-#
-# Revision 1.38  2005/03/08 16:42:47  ncq
-# - there are episodes w/ and w/o fk_patient IS NULL so handle that
-#   properly in set_active()
-#
-# Revision 1.37  2005/02/28 18:15:36  ncq
-# - proper fix for not being able to fetch unnamed episodes
-#   is to require a name in the first place ...
-#
-# Revision 1.36  2005/02/20 10:30:49  sjtan
-#
-# unnamed episodes cannot be refetched.
-#
-# Revision 1.35  2005/01/31 12:58:24  ncq
-# - episode.rename() finally works
-#
-# Revision 1.34  2005/01/31 09:35:28  ncq
-# - add episode.get_description() to return clin_narrative row
-# - improve episode.rename() - works for adding new narrative now
-# - improve create_episode() - revise later
-# - improve unit testing
-#
-# Revision 1.33  2005/01/29 17:53:57  ncq
-# - debug/enhance create_episode
-#
-# Revision 1.32  2005/01/25 17:24:57  ncq
-# - streamlined cEpisode.rename()
-#
-# Revision 1.31  2005/01/25 01:36:19  cfmoro
-# Added cEpisode.rename method
-#
-# Revision 1.30  2005/01/15 20:24:35  ncq
-# - streamlined cProblem
-#
-# Revision 1.29  2005/01/15 19:55:55  cfmoro
-# Added problem support to emr
-#
-# Revision 1.28  2005/01/02 19:55:30  ncq
-# - don't need _xmins_refetch_col_pos anymore
-#
-# Revision 1.27  2004/12/20 16:45:49  ncq
-# - gmBusinessDBObject now requires refetching of XMIN after save_payload
-#
-# Revision 1.26  2004/12/15 10:42:09  ncq
-# - cClinEpisode not handles the fields properly
-#
-# Revision 1.25  2004/12/15 10:28:11  ncq
-# - fix create_episode() aka add_episode()
-#
-# Revision 1.24  2004/11/03 22:32:34  ncq
-# - support _cmds_lock_rows_for_update in business object base class
-#
-# Revision 1.23  2004/09/19 15:02:29  ncq
-# - episode: id -> pk, support fk_patient
-# - no default name in create_health_issue
-#
-# Revision 1.22  2004/07/05 10:24:46  ncq
-# - use v_pat_rfe/aoe, by Carlos
-#
-# Revision 1.21  2004/07/04 15:09:40  ncq
-# - when refactoring need to fix imports, too
-#
-# Revision 1.20  2004/07/04 13:24:31  ncq
-# - add cRFE/cAOE
-# - use in get_rfes(), get_aoes()
-#
-# Revision 1.19  2004/06/30 20:34:37  ncq
-# - cEncounter.get_RFEs()
-#
-# Revision 1.18  2004/06/26 23:45:50  ncq
-# - cleanup, id_* -> fk/pk_*
-#
-# Revision 1.17  2004/06/26 07:33:55  ncq
-# - id_episode -> fk/pk_episode
-#
-# Revision 1.16  2004/06/08 00:44:41  ncq
-# - v_pat_episodes now has description, not episode for name of episode
-#
-# Revision 1.15  2004/06/02 22:12:48  ncq
-# - cleanup
-#
-# Revision 1.14  2004/06/02 13:45:19  sjtan
-#
-# episode->description for update statement as well.
-#
-# Revision 1.13  2004/06/02 13:18:48  sjtan
-#
-# revert, as backend view definition was changed yesterday to be more consistent.
-#
-# Revision 1.12  2004/06/02 12:48:56  sjtan
-#
-# map episode to description in cursor.description, so can find as episode['description']
-# and also save.
-#
-# Revision 1.11  2004/06/01 23:53:56  ncq
-# - v_pat_episodes.episode -> *.description
-#
-# Revision 1.10  2004/06/01 08:20:14  ncq
-# - limit in get_lab_results
-#
-# Revision 1.9  2004/05/30 20:10:31  ncq
-# - cleanup
-#
-# Revision 1.8  2004/05/22 12:42:54  ncq
-# - add create_episode()
-# - cleanup add_episode()
-#
-# Revision 1.7  2004/05/18 22:36:52  ncq
-# - need mx.DateTime
-# - fix fields updatable in episode
-# - fix delete action in episode.set_active()
-#
-# Revision 1.6  2004/05/18 20:35:42  ncq
-# - cleanup
-#
-# Revision 1.5  2004/05/17 19:02:26  ncq
-# - encounter.set_active()
-# - improve create_encounter()
-#
-# Revision 1.4  2004/05/16 15:47:51  ncq
-# - add episode.set_active()
-#
-# Revision 1.3  2004/05/16 14:31:27  ncq
-# - cleanup
-# - allow health issue to be instantiated by name/patient
-# - create_health_issue()/create_encounter
-# - based on Carlos' work
-#
-# Revision 1.2  2004/05/12 14:28:53  ncq
-# - allow dict style pk definition in __init__ for multicolum primary keys (think views)
-# - self.pk -> self.pk_obj
-# - __init__(aPKey) -> __init__(aPK_obj)
-#
-# Revision 1.1  2004/04/17 12:18:50  ncq
-# - health issue, episode, encounter classes
-#
+
