@@ -4,9 +4,7 @@
 license: GPL
 """
 #============================================================
-# $Source: /home/ncq/Projekte/cvs2git/vcs-mirror/gnumed/gnumed/client/business/gmMedication.py,v $
-# $Id: gmMedication.py,v 1.20 2009-12-25 21:38:50 ncq Exp $
-__version__ = "$Revision: 1.20 $"
+__version__ = "$Revision: 1.21 $"
 __author__ = "K.Hilbert <Karsten.Hilbert@gmx.net>"
 
 import sys, logging, csv, codecs, os, re as regex
@@ -14,13 +12,52 @@ import sys, logging, csv, codecs, os, re as regex
 
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
-from Gnumed.pycommon import gmBusinessDBObject, gmPG2, gmShellAPI, gmTools, gmDateTime
-from Gnumed.business import gmATC
+from Gnumed.pycommon import gmBusinessDBObject, gmPG2, gmShellAPI, gmTools
+from Gnumed.pycommon import gmDispatcher, gmDateTime, gmHooks
+from Gnumed.business import gmATC, gmAllergy
 
 
 _log = logging.getLogger('gm.meds')
 _log.info(__version__)
 
+#============================================================
+def _on_substance_intake_modified():
+	"""Always relates to the active patient."""
+	gmHooks.run_hook_script(hook = u'after_substance_intake_modified')
+
+gmDispatcher.connect(_on_substance_intake_modified, u'substance_intake_mod_db')
+
+#============================================================
+# this should be in gmCoding.py
+def create_data_source(long_name=None, short_name=None, version=None, source=None, language=None):
+
+		args = {
+			'lname': long_name,
+			'sname': short_name,
+			'ver': version,
+			'src': source,
+			'lang': language
+		}
+
+		cmd = u"""select pk from ref.data_source where name_long = %(lname)s and name_short = %(sname)s and version = %(ver)s"""
+		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}])
+		if len(rows) > 0:
+			return rows[0]['pk']
+
+		cmd = u"""
+			INSERT INTO ref.data_source (name_long, name_short, version, source, lang)
+			VALUES (
+				%(lname)s,
+				%(sname)s,
+				%(ver)s,
+				%(src)s,
+				%(lang)s
+			)
+			returning pk
+			"""
+		rows, idx = gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}], return_data = True)
+
+		return rows[0]['pk']
 #============================================================
 # wishlist:
 # - --conf-file= for glwin.exe
@@ -75,7 +112,7 @@ class cGelbeListeCSVFile(object):
 		u'status_billigere_packung',
 		u'rezepttyp',
 		u'besonderes_arzneimittel',			# Abstimmungsverfahren SGB-V
-		u't-rezept-pflicht',				# Thalidomid-Rezept
+		u't_rezept_pflicht',				# Thalidomid-Rezept
 		u'erstattbares_medizinprodukt',
 		u'hilfsmittel',
 		u'hzv_rabattkennung',
@@ -95,7 +132,7 @@ class cGelbeListeCSVFile(object):
 		u'apothekenpflicht',
 		u'status_billigere_packung',
 		u'besonderes_arzneimittel',			# Abstimmungsverfahren SGB-V
-		u't-rezept-pflicht',
+		u't_rezept_pflicht',
 		u'erstattbares_medizinprodukt',
 		u'hilfsmittel'
 	]
@@ -148,13 +185,17 @@ class cGelbeListeCSVFile(object):
 class cDrugDataSourceInterface(object):
 
 	#--------------------------------------------------------
+	def __init__(self):
+		self.patient = None
+		self.custom_path_to_binary = None
+	#--------------------------------------------------------
 	def get_data_source_version(self):
 		raise NotImplementedError
 	#--------------------------------------------------------
 	def create_data_source_entry(self):
 		raise NotImplementedError
 	#--------------------------------------------------------
-	def switch_to_frontend(self):
+	def switch_to_frontend(self, blocking=False):
 		raise NotImplementedError
 	#--------------------------------------------------------
 	def select_drugs(self):
@@ -163,12 +204,122 @@ class cDrugDataSourceInterface(object):
 	def import_drugs(self):
 		raise NotImplementedError
 	#--------------------------------------------------------
-	def check_drug_interactions(self):
+	def check_drug_interactions(self, drug_ids_list=None, substances=None):
 		raise NotImplementedError
 	#--------------------------------------------------------
 	def show_info_on_drug(self, drug=None):
 		raise NotImplementedError
 #============================================================
+class cFreeDiamsInterface(cDrugDataSourceInterface):
+
+	"""http://ericmaeker.fr/FreeMedForms/di-manual/ligne_commandes.html"""
+
+	version = u'FreeDiams v0.3.0 interface'
+	default_encoding = 'utf8'
+	default_dob_format = '%d/%m/%Y'
+
+	map_gender2mf = {
+		'm': u'M',
+		'f': u'F',
+		'tf': u'F',
+		'tm': u'M',
+		'h': u'H'
+	}
+	#--------------------------------------------------------
+	def __init__(self):
+		cDrugDataSourceInterface.__init__(self)
+		_log.info(cFreeDiamsInterface.version)
+
+		paths = gmTools.gmPaths()
+		self.__exchange_filename = os.path.join(paths.home_dir, '.gnumed', 'tmp', 'gm2freediams.xml')
+	#--------------------------------------------------------
+	def get_data_source_version(self):
+		#> Coded. Available next release
+		#> Use --version or -version or -v
+		return u'0.3.0'
+		# ~/.freediams/config.ini: [License] -> AcceptedVersion=....
+	#--------------------------------------------------------
+	def create_data_source_entry(self):
+		return create_data_source (
+			long_name = u'"FreeDiams" Drug Database Frontend',
+			short_name = u'FreeDiams',
+			version = self.get_data_source_version(),
+			source = u'http://ericmaeker.fr/FreeMedForms/di-manual/index.html',
+			language = u'fr'			# actually to be multi-locale
+		)
+	#--------------------------------------------------------
+	def switch_to_frontend(self, blocking=False):
+		"""	--medintux : définit une utilisation spécifique à MedinTux.
+			--exchange="xxx" : définit le fichier d'échange entre les deux applications.
+			--chrono : Chronomètres diverses fonctions du testeur d'interactions (proposé à des fins de déboggage)
+			--transmit-dosage = non documenté.
+		"""
+		found, cmd = gmShellAPI.find_first_binary(binaries = [
+			self.custom_path_to_binary,
+			r'/usr/bin/freediams',
+			r'freediams',
+			r'/Applications/FreeDiams.app/Contents/MacOs/FreeDiams',
+			r'c:\programs\freediams\freediams.exe',
+			r'freediams.exe'
+		])
+
+		if not found:
+			_log.error('cannot find FreeDiams binary')
+			return False
+
+		# make sure csv file exists
+		open(self.__exchange_filename, 'wb').close()
+		args = u'--exchange="%s"' % self.__exchange_filename
+
+		if self.patient is not None:
+
+			args += u' --patientname="%(firstnames)s %(lastnames)s"' % self.patient.get_active_name()
+
+			args += u' --gender=%s' % cFreeDiamsInterface.map_gender2mf[self.patient['gender']]
+
+			if self.patient['dob'] is not None:
+				args += u' --dateofbirth="%s"' % self.patient['dob'].strftime(cFreeDiamsInterface.default_dob_format)
+
+			# FIXME: search by LOINC code and add
+			# --weight="dd" : définit le poids du patient (en kg)
+			# --size="ddd" : définit la taille du patient (en cm)
+			# --clcr="dd.d" : définit la clairance de la créatinine du patient (en ml/min)
+			# --creatinin="dd" : définit la créatininémie du patient (en mg/l)
+
+		cmd = r'%s %s' % (cmd, args)
+
+		if not gmShellAPI.run_command_in_shell(command = cmd):
+			_log.error('problem switching to the FreeDiams drug database')
+			return False
+
+		return True
+	#--------------------------------------------------------
+	def select_drugs(self):
+		self.switch_to_frontend()
+	#--------------------------------------------------------
+	def import_drugs(self):
+		"""FreeDiams ONLY use CIS.
+
+			CIS stands for Unique Speciality Identifier (eg bisoprolol 5 mg, gel).
+			CIS is AFSSAPS specific, but pharmacist can retreive drug name with the CIS.
+			AFSSAPS is the French FDA.
+
+			CIP stands for Unique Presentation Identifier (eg 30 pills plaq)
+			CIP if you want to specify the packaging of the drug (30 pills
+			thermoformed tablet...) -- actually not really usefull for french
+			doctors.
+		"""
+		self.switch_to_frontend()
+		# .external_code_type: u'FR-CIS'
+		# .external_cod: the CIS value
+	#--------------------------------------------------------
+	def check_drug_interactions(self, drug_ids_list=None, substances=None):
+		self.switch_to_frontend()
+	#--------------------------------------------------------
+	def show_info_on_drug(self, drug=None):
+		# pass in CIS
+		self.switch_to_frontend()
+#============================================================
 class cGelbeListeWindowsInterface(cDrugDataSourceInterface):
 	"""Support v8.2 CSV file interface only."""
 
@@ -178,6 +329,9 @@ class cGelbeListeWindowsInterface(cDrugDataSourceInterface):
 	bdt_line_base_length = 8
 	#--------------------------------------------------------
 	def __init__(self):
+
+		cDrugDataSourceInterface.__init__(self)
+
 		_log.info(u'%s (native Windows)', cGelbeListeWindowsInterface.version)
 
 		self.path_to_binary = r'C:\Programme\MMI PHARMINDEX\glwin.exe'
@@ -190,63 +344,63 @@ class cGelbeListeWindowsInterface(cDrugDataSourceInterface):
 		self.interactions_filename = os.path.join(paths.home_dir, '.gnumed', 'tmp', 'gm2mmi.bdt')
 		self.data_date_filename = r'C:\Programme\MMI PHARMINDEX\datadate.txt'
 
-		self.data_date = None
-		self.online_update_date = None
+		self.__data_date = None
+		self.__online_update_date = None
 
 		# use adjusted config.dat
 	#--------------------------------------------------------
-	def get_data_source_version(self):
+	def get_data_source_version(self, force_reload=False):
+
+		if self.__data_date is not None:
+			if not force_reload:
+				return {
+					'data': self.__data_date,
+					'online_update': self.__online_update_date
+				}
 
 		open(self.data_date_filename, 'wb').close()
 
 		cmd = u'%s -DATADATE' % self.path_to_binary
 		if not gmShellAPI.run_command_in_shell(command = cmd, blocking = True):
 			_log.error('problem querying the MMI drug database for version information')
+			self.__data_date = None
+			self.__online_update_date = None
 			return {
 				'data': u'?',
 				'online_update': u'?'
 			}
 
-		version_file = open(self.data_date_filename, 'rU')
-		versions = {
-			'data': version_file.readline()[:10],
-			'online_update': version_file.readline()[:10]
-		}
+		try:
+			version_file = open(self.data_date_filename, 'rU')
+		except StandardError:
+			_log.error('problem querying the MMI drug database for version information')
+			_log.exception('cannot open MMI drug database version file [%s]', self.data_date_filename)
+			self.__data_date = None
+			self.__online_update_date = None
+			return {
+				'data': u'?',
+				'online_update': u'?'
+			}
+
+		self.__data_date = version_file.readline()[:10]
+		self.__online_update_date = version_file.readline()[:10]
 		version_file.close()
 
-		return versions
+		return {
+			'data': self.__data_date,
+			'online_update': self.__online_update_date
+		}
 	#--------------------------------------------------------
 	def create_data_source_entry(self):
 		versions = self.get_data_source_version()
 
-		args = {
-			'lname': u'Medikamentendatenbank "mmi PHARMINDEX" (Gelbe Liste)',
-			'sname': u'GL/MMI',
-			'ver': u'Daten: %s, Preise (Onlineupdate): %s' % (versions['data'], versions['online_update']),
-			'src': u'Medizinische Medien Informations GmbH, Am Forsthaus Gravenbruch 7, 63263 Neu-Isenburg',
-			'lang': u'de'
-		}
-
-		cmd = u"""select pk from ref.data_source where name_long = %(lname)s and name_short = %(sname)s and version = %(ver)s"""
-		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}])
-		if len(rows) > 0:
-			return rows[0]['pk']
-
-		cmd = u"""
-			INSERT INTO ref.data_source (name_long, name_short, version, source, lang)
-			VALUES (
-				%(lname)s,
-				%(sname)s,
-				%(ver)s,
-				%(src)s,
-				%(lang)s
-			)
-			returning pk
-			"""
-
-		rows, idx = gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}], return_data = True)
-
-		return rows[0]['pk']
+		return create_data_source (
+			long_name = u'Medikamentendatenbank "mmi PHARMINDEX" (Gelbe Liste)',
+			short_name = u'GL/MMI',
+			version = u'Daten: %s, Preise (Onlineupdate): %s' % (versions['data'], versions['online_update']),
+			source = u'Medizinische Medien Informations GmbH, Am Forsthaus Gravenbruch 7, 63263 Neu-Isenburg',
+			language = u'de'
+		)
 	#--------------------------------------------------------
 	def switch_to_frontend(self, blocking=False, cmd=None):
 
@@ -325,7 +479,8 @@ class cGelbeListeWindowsInterface(cDrugDataSourceInterface):
 			# update fields
 			drug['is_fake'] = False
 			drug['atc_code'] = entry['atc']
-			drug['external_code'] = u'%s::%s' % ('DE-PZN', entry['pzn'])
+			drug['external_code_type'] = u'DE-PZN'
+			drug['external_code'] = entry['pzn']
 			drug['fk_data_source'] = data_src_pk
 			drug.save()
 
@@ -345,28 +500,27 @@ class cGelbeListeWindowsInterface(cDrugDataSourceInterface):
 
 		return new_drugs, new_substances
 	#--------------------------------------------------------
-	def check_drug_interactions(self, pzn_list=None, substances=None):
+	def check_drug_interactions(self, drug_ids_list=None, substances=None):
 		"""For this to work the BDT interaction check must be configured in the MMI."""
 
-		if pzn_list is None:
+		if drug_ids_list is None:
 			if substances is None:
 				return
 			if len(substances) < 2:
 				return
-			pzn_list = [ s.external_code for s in substances ]
-			pzn_list = [ pzn for pzn in pzn_list if pzn is not None ]
-			pzn_list = [ code_value for code_type, code_value in pzn_list if code_type == u'DE-PZN']
+			drug_ids_list = [ (s.external_code_type, s.external_code) for s in substances ]
+			drug_ids_list = [ code_value for code_type, code_value in drug_ids_list if (code_value is not None) and (code_type == u'DE-PZN')]
 
 		else:
-			if len(pzn_list) < 2:
+			if len(drug_ids_list) < 2:
 				return
 
-		if pzn_list < 2:
+		if drug_ids_list < 2:
 			return
 
 		bdt_file = codecs.open(filename = self.interactions_filename, mode = 'wb', encoding = cGelbeListeWindowsInterface.default_encoding)
 
-		for pzn in pzn_list:
+		for pzn in drug_ids_list:
 			pzn = pzn.strip()
 			lng = cGelbeListeWindowsInterface.bdt_line_base_length + len(pzn)
 			bdt_file.write(cGelbeListeWindowsInterface.bdt_line_template % (lng, pzn))
@@ -379,10 +533,8 @@ class cGelbeListeWindowsInterface(cDrugDataSourceInterface):
 
 		cmd = None
 
-		if substance.external_code is not None:
-			code_type, pzn = substance.external_code
-			if code_type == u'DE-PZN':
-				cmd = u'%s -PZN %s' % (self.path_to_binary, pzn)
+		if substance.external_code_type == u'DE-PZN':
+			cmd = u'%s -PZN %s' % (self.path_to_binary, substance.external_code)
 
 		if cmd is None:
 			name = gmTools.coalesce (
@@ -459,8 +611,9 @@ class cIfapInterface(cDrugDataSourceInterface):
 #				)
 #============================================================
 drug_data_source_interfaces = {
-	'Gelbe Liste/MMI (Windows)': cGelbeListeWindowsInterface,
-	'Gelbe Liste/MMI (WINE)': cGelbeListeWineInterface
+	'Deutschland: Gelbe Liste/MMI (Windows)': cGelbeListeWindowsInterface,
+	'Deutschland: Gelbe Liste/MMI (WINE)': cGelbeListeWineInterface,
+	'France: FreeDiams': cFreeDiamsInterface
 }
 #============================================================
 # substances in use across all patients
@@ -519,6 +672,8 @@ class cSubstanceIntakeEntry(gmBusinessDBObject.cBusinessDBObject):
 	_cmds_store_payload = [
 		u"""update clin.substance_intake set
 				clin_when = %(started)s,
+				discontinued = %(discontinued)s,
+				discontinue_reason = gm.nullify_empty_string(%(discontinue_reason)s),
 				strength = gm.nullify_empty_string(%(strength)s),
 				preparation = %(preparation)s,
 				schedule = gm.nullify_empty_string(%(schedule)s),
@@ -532,7 +687,7 @@ class cSubstanceIntakeEntry(gmBusinessDBObject.cBusinessDBObject):
 						when (
 							(%(is_long_term)s is False)
 								and
-							(gm.is_null_or_blank_string(%(duration)s) is True)
+							(%(duration)s IS NULL)
 						) is True then null
 						else %(is_long_term)s
 					end
@@ -540,7 +695,7 @@ class cSubstanceIntakeEntry(gmBusinessDBObject.cBusinessDBObject):
 				duration = (
 					case
 						when %(is_long_term)s is True then null
-						else gm.nullify_empty_string(%(duration)s)
+						else %(duration)s
 					end
 				)::interval,
 
@@ -556,6 +711,8 @@ class cSubstanceIntakeEntry(gmBusinessDBObject.cBusinessDBObject):
 	]
 	_updatable_fields = [
 		u'started',
+		u'discontinued',
+		u'discontinue_reason',
 		u'preparation',
 		u'strength',
 		u'intake_is_approved_of',
@@ -597,6 +754,51 @@ class cSubstanceIntakeEntry(gmBusinessDBObject.cBusinessDBObject):
 
 		return line
 	#--------------------------------------------------------
+	def turn_into_allergy(self, encounter_id=None, allergy_type='allergy'):
+		allg = gmAllergy.create_allergy (
+			allergene = self._payload[self._idx['substance']],
+			allg_type = allergy_type,
+			episode_id = self._payload[self._idx['pk_episode']],
+			encounter_id = encounter_id
+		)
+		allg['substance'] = gmTools.coalesce (
+			self._payload[self._idx['brand']],
+			self._payload[self._idx['substance']]
+		)
+		allg['reaction'] = self._payload[self._idx['discontinue_reason']]
+		allg['atc_code'] = gmTools.coalesce(self._payload[self._idx['atc_substance']], self._payload[self._idx['atc_brand']])
+		if self._payload[self._idx['external_code_brand']] is not None:
+			allg['substance_code'] = u'%s::::%s' % (self._payload[self._idx['external_code_type_brand']], self._payload[self._idx['external_code_brand']])
+		allg['allergene'] = self._payload[self._idx['substance']]
+		allg['generics'] = self._payload[self._idx['substance']]
+
+		allg.save()
+		return allg
+	#--------------------------------------------------------
+	# properties
+	#--------------------------------------------------------
+	def _get_ddd(self):
+
+		try: self.__ddd
+		except AttributeError: self.__ddd = None
+
+		if self.__ddd is not None:
+			return self.__ddd
+
+		if self._payload[self._idx['atc_substance']] is not None:
+			ddd = gmATC.atc2ddd(atc = self._payload[self._idx['atc_substance']])
+			if len(ddd) != 0:
+				self.__ddd = ddd[0]
+		else:
+			if self._payload[self._idx['atc_brand']] is not None:
+				ddd = gmATC.atc2ddd(atc = self._payload[self._idx['atc_brand']])
+				if len(ddd) != 0:
+					self.__ddd = ddd[0]
+
+		return self.__ddd
+
+	ddd = property(_get_ddd, lambda x:x)
+	#--------------------------------------------------------
 	def _get_external_code(self):
 		drug = self.containing_drug
 
@@ -607,6 +809,16 @@ class cSubstanceIntakeEntry(gmBusinessDBObject.cBusinessDBObject):
 
 	external_code = property(_get_external_code, lambda x:x)
 	#--------------------------------------------------------
+	def _get_external_code_type(self):
+		drug = self.containing_drug
+
+		if drug is None:
+			return None
+
+		return drug.external_code_type
+
+	external_code_type = property(_get_external_code_type, lambda x:x)
+	#--------------------------------------------------------
 	def _get_containing_drug(self):
 		if self._payload[self._idx['pk_brand']] is None:
 			return None
@@ -671,6 +883,7 @@ class cBrandedDrug(gmBusinessDBObject.cBusinessDBObject):
 				preparation = %(preparation)s,
 				atc_code = gm.nullify_empty_string(%(atc_code)s),
 				external_code = gm.nullify_empty_string(%(external_code)s),
+				external_code_type = gm.nullify_empty_string(%(external_code_type)s),
 				is_fake = %(is_fake)s,
 				fk_data_source = %(fk_data_source)s
 			where
@@ -686,6 +899,7 @@ class cBrandedDrug(gmBusinessDBObject.cBusinessDBObject):
 		u'atc_code',
 		u'is_fake',
 		u'external_code',
+		u'external_code_type',
 		u'fk_data_source'
 	]
 	#--------------------------------------------------------
@@ -693,14 +907,20 @@ class cBrandedDrug(gmBusinessDBObject.cBusinessDBObject):
 		if self._payload[self._idx['external_code']] is None:
 			return None
 
-		if regex.match(u'.+::.+', self._payload[self._idx['external_code']], regex.UNICODE) is None:
-			# FIXME: maybe evaluate fk_data_source
-			return None
-
-		return regex.split(u'::', self._payload[self._idx['external_code']], 1)
+		return self._payload[self._idx['external_code']]
 
 	external_code = property(_get_external_code, lambda x:x)
 	#--------------------------------------------------------
+	def _get_external_code_type(self):
+
+		# FIXME: maybe evaluate fk_data_source ?
+		if self._payload[self._idx['external_code_type']] is None:
+			return None
+
+		return self._payload[self._idx['external_code_type']]
+
+	external_code_type = property(_get_external_code_type, lambda x:x)
+	#--------------------------------------------------------
 	def _get_components(self):
 		cmd = u'select * from ref.substance_in_brand where fk_brand = %(brand)s'
 		args = {'brand': self._payload[self._idx['pk']]}
@@ -799,6 +1019,12 @@ def delete_component_from_branded_drug(brand=None, component=None):
 #------------------------------------------------------------
 if __name__ == "__main__":
 
+	if len(sys.argv) < 2:
+		sys.exit()
+
+	if sys.argv[1] != 'test':
+		sys.exit()
+
 	from Gnumed.pycommon import gmLog2
 	from Gnumed.pycommon import gmI18N
 
@@ -847,7 +1073,7 @@ if __name__ == "__main__":
 		# Metoprolol + Hct vs Citalopram
 		diclofenac = '7587712'
 		phenprocoumon = '4421744'
-		mmi.check_drug_interactions(pzn_list = [diclofenac, phenprocoumon])
+		mmi.check_drug_interactions(drug_ids_list = [diclofenac, phenprocoumon])
 	#--------------------------------------------------------
 	def test_create_substance_intake():
 		drug = create_substance_intake (
@@ -864,101 +1090,13 @@ if __name__ == "__main__":
 		print drug
 		print drug.components
 	#--------------------------------------------------------
-	if (len(sys.argv)) > 1 and (sys.argv[1] == 'test'):
-		#test_MMI_interface()
-		#test_MMI_file()
-		#test_mmi_switch_to()
-		#test_mmi_select_drugs()
-		#test_mmi_import_substances()
-		#test_mmi_import_drugs()
-		#test_interaction_check()
-		#test_create_substance_intake()
-		test_show_components()
+	#test_MMI_interface()
+	#test_MMI_file()
+	#test_mmi_switch_to()
+	#test_mmi_select_drugs()
+	#test_mmi_import_substances()
+	#test_mmi_import_drugs()
+	#test_interaction_check()
+	#test_create_substance_intake()
+	test_show_components()
 #============================================================
-# $Log: gmMedication.py,v $
-# Revision 1.20  2009-12-25 21:38:50  ncq
-# - add 2 new fields to MMI CSV file
-# - show-info-on-drug
-# - enhance switch-to-frontend to allow custom startup cmd
-#
-# Revision 1.19  2009/12/02 16:48:58  ncq
-# - add infrastructure for removing component from brand
-#
-# Revision 1.18  2009/12/01 21:48:09  ncq
-# - get-substance-by-pk
-#
-# Revision 1.17  2009/11/30 21:56:36  ncq
-# - components property on branded drug
-#
-# Revision 1.16  2009/11/30 15:06:27  ncq
-# - handle a bunch of possibilities of dirty records retrieved from GLI/MMI
-# - default preparation to i18n(units)
-#
-# Revision 1.15  2009/11/29 19:59:31  ncq
-# - improve substance/component creation with propagate-atc
-#
-# Revision 1.14  2009/11/29 15:57:27  ncq
-# - while SQL results are dicts as far as *retrieval* is concerned,
-#   they are NOT for inserting data into them, so use list access
-#
-# Revision 1.13  2009/11/28 18:27:30  ncq
-# - much improved ATC detection on substance creation
-# - create-patient-consumed-substance -> create-substance-intake
-# - get-branded-drugs
-# - get-substances-in-brands
-# - delete-branded-drugs
-#
-# Revision 1.12  2009/11/24 19:57:22  ncq
-# - implement getting/creating data souce entry for MMI
-# - implement version retrieval for MMI
-# - import-drugs()
-# - check-drug-interactions()
-# - cConsumedSubstance -> cSubstanceIntakeEntry + .external_code
-# - cBrandedDrug
-# - tests
-#
-# Revision 1.11  2009/11/06 15:05:07  ncq
-# - get-substances-in-use
-# - meds formatting
-# - delete-patient-consumed-substance
-#
-# Revision 1.10  2009/10/29 17:16:59  ncq
-# - return newly created substances from creator func and substance importer method
-# - better naming
-# - finish up cConsumedSubstance
-#
-# Revision 1.9  2009/10/28 16:40:12  ncq
-# - add some docs about schedule parsing
-#
-# Revision 1.8  2009/10/26 22:29:05  ncq
-# - better factorization of paths in MMI interface
-# - update ATC on INN if now known
-# - delete-consumed-substance
-#
-# Revision 1.7  2009/10/21 20:37:18  ncq
-# - MMI uses cp1250, rather than cp1252, (at least under WINE) contrary to direct communication ...
-# - use unicode csv reader
-# - add a bunch of file cleanup
-# - split MMI interface into WINE vs native Windows version
-#
-# Revision 1.6  2009/10/21 09:15:50  ncq
-# - much improved MMI frontend
-#
-# Revision 1.5  2009/09/29 13:14:25  ncq
-# - faulty ordering of definitions
-#
-# Revision 1.4  2009/09/01 22:16:35  ncq
-# - improved interaction check test
-#
-# Revision 1.3  2009/08/24 18:36:20  ncq
-# - add CSV file iterator
-# - add BDT interaction check
-#
-# Revision 1.2  2009/08/21 09:56:37  ncq
-# - start drug data source interfaces
-# - add MMI/Gelbe Liste interface
-#
-# Revision 1.1  2009/05/12 12:02:01  ncq
-# - start supporting current medications
-#
-#
\ No newline at end of file
