@@ -25,7 +25,7 @@ import mx.DateTime as mxDT
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
 from Gnumed.pycommon import gmI18N, gmExceptions, gmNull, gmPG2, gmTools
-from Gnumed.business import gmClinicalRecord, gmPerson, gmAllergy, gmMedDoc, gmDemographicRecord, gmClinNarrative
+from Gnumed.business import gmClinicalRecord, gmPerson, gmAllergy, gmDemographicRecord, gmClinNarrative
 
 
 _log = logging.getLogger('gm.export')
@@ -834,8 +834,7 @@ class cEmrExport:
                 doc['l10n_type'],
                 doc['comment'])
             )
-            parts = doc.get_parts()
-            for part in parts:
+            for part in doc.parts:
                 self.__target.write('\n         %s - %s' % (
                     part['seq_idx'],
                     part['obj_comment'])
