@@ -197,6 +197,10 @@ def init():
 def pydt_now_here():
 	"""Returns NOW @ HERE (IOW, in the local timezone."""
 	return pyDT.datetime.now(gmCurrentLocalTimezone)
+#---------------------------------------------------------------------------
+def wx_now_here(wx=None):
+	"""Returns NOW @ HERE (IOW, in the local timezone."""
+	return py_dt2wxDate(py_dt = pydt_now_here(), wx = wx)
 #===========================================================================
 # wxPython conversions
 #---------------------------------------------------------------------------
@@ -232,10 +236,11 @@ def wxDate2py_dt(wxDate=None):
 		raise
 #---------------------------------------------------------------------------
 def py_dt2wxDate(py_dt=None, wx=None):
-	wxdt = wx.DateTime()
-	wxdt.SetYear(py_dt.year)
-	wxdt.SetMonth(py_dt.month-1)
-	wxdt.SetDay(py_dt.day)
+	_log.debug(u'setting wx.DateTime from: %s-%s-%s', py_dt.year, py_dt.month, py_dt.day)
+	# Robin Dunn says that for SetYear/*Month/*Day the wx.DateTime MUST already
+	# be valid (by definition) or, put the other way round, you must Set() day,
+	# month, and year at once
+	wxdt = wx.DateTimeFromDMY(py_dt.day, py_dt.month-1, py_dt.year)
 	return wxdt
 #===========================================================================
 # interval related
@@ -1514,4 +1519,4 @@ if __name__ == '__main__':
 #
 # Revision 1.1  2006/05/22 12:00:00  ncq
 # - first cut at this
-#
\ No newline at end of file
+#
