@@ -6,9 +6,6 @@ phrasewheels the ability to guess phrases.
 Copyright (C) GNUMed developers
 license: GPL
 """
-############################################################################
-# $Source: /home/ncq/Projekte/cvs2git/vcs-mirror/gnumed/gnumed/client/pycommon/gmMatchProvider.py,v $
-# $Id: gmMatchProvider.py,v 1.34 2009-12-21 15:02:17 ncq Exp $
 __version__ = "$Revision: 1.34 $"
 __author__  = "K.Hilbert <Karsten.Hilbert@gmx.net>, I.Haywood <ihaywood@gnu.org>, S.J.Tan <sjtan@bigpond.com>"
 
@@ -440,7 +437,7 @@ class cMatchProvider_SQL2(cMatchProvider):
 			try:
 				rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': self._args}])
 			except:
-				_log.exception('Error running match provider SQL, dropping query.')
+				_log.exception('[%s]: error running match provider SQL, dropping query', self.__class__.__name__)
 				idx = self._queries.index(query)
 				del self._queries[idx]
 				break
@@ -460,147 +457,3 @@ if __name__ == '__main__':
 	pass
 
 #================================================================
-# $Log: gmMatchProvider.py,v $
-# Revision 1.34  2009-12-21 15:02:17  ncq
-# - fix typo
-#
-# Revision 1.33  2009/04/05 17:58:27  ncq
-# - improved docs
-#
-# Revision 1.32  2009/04/03 09:34:06  ncq
-# - some exception cleanup
-#
-# Revision 1.31  2009/03/01 18:07:14  ncq
-# - factor out default ignored chars/word separators onto module level
-#
-# Revision 1.30  2009/01/21 22:34:09  ncq
-# - make FixedList match provider work nicely again
-#
-# Revision 1.29  2008/06/16 15:02:35  ncq
-# - cleanup
-# - remove unneeded methods
-#
-# Revision 1.28  2008/06/15 20:31:10  ncq
-# - make match provider derive from object
-# - turn ignored chars and word separators into properties
-# - raise NotImplementedError in base match provider
-# - remove dis/enableMatching
-#
-# Revision 1.27  2008/06/09 15:28:21  ncq
-# - .print_queries and support it in sql provider
-#
-# Revision 1.26  2008/04/29 18:29:29  ncq
-# - remove increaseScore
-#
-# Revision 1.25  2007/12/12 16:17:15  ncq
-# - better logger names
-#
-# Revision 1.24  2007/12/11 14:31:11  ncq
-# - use std logging
-#
-# Revision 1.23  2007/12/02 20:59:13  ncq
-# - drop failing queries
-#
-# Revision 1.22  2007/07/03 15:57:24  ncq
-# - use gmPG2.sanitize_pg_regex()
-# - ignore failing match retrieval queries such
-#   that we don't freak out in the phrasewheel
-#
-# Revision 1.21  2007/01/07 23:02:11  ncq
-# - more documentation on context
-#
-# Revision 1.20  2006/11/06 09:59:42  ncq
-# - when allowing non-list strings to turn into query list do not
-#   str() them or else we may lose unicodity
-# - more u''ing
-#
-# Revision 1.19  2006/11/05 16:07:31  ncq
-# - *_SQL2 now really handles context values, tested, too
-# - some u''-ification
-# - don't sort items in *_SQL2, rely on in-query ORDER BY instead
-#
-# Revision 1.18  2006/10/24 13:18:29  ncq
-# - switch to gmPG2
-# - remove cMatchProvider_SQL()
-#
-# Revision 1.17  2006/05/25 22:13:30  ncq
-# - robustify set_context()
-#
-# Revision 1.16  2006/05/01 18:46:05  ncq
-# - cleanup
-#
-# Revision 1.15  2005/06/14 18:54:40  ncq
-# - don't sort in SQL2 matcher - queries should ORDER BY
-#
-# Revision 1.14  2005/06/12 21:20:55  ncq
-# - make SQL2 match provider more robust regarding query list
-#
-# Revision 1.13  2005/06/12 21:16:55  ncq
-# - make SQL2 match provider accept a query list
-#
-# Revision 1.12  2005/06/10 17:07:34  cfmoro
-# Fixed set_context in SQL2
-#
-# Revision 1.11  2005/06/08 01:27:12  cfmoro
-# Renamed function to make parent set_context work
-#
-# Revision 1.10  2005/06/07 10:16:37  ncq
-# - setContext -> set_context
-#
-# Revision 1.9  2005/05/08 21:40:57  ncq
-# - cleanup
-#
-# Revision 1.8  2005/04/14 18:24:57  ncq
-# - some cleanup of funky magic so we are faster
-#
-# Revision 1.7  2005/04/11 18:00:54  ncq
-# - cleanup
-#
-# Revision 1.6  2005/03/14 14:35:27  ncq
-# - add match provider class cMatchProvider_Func which pulls
-#   match candidates through a function
-#
-# Revision 1.5  2004/07/17 21:08:51  ncq
-# - gmPG.run_query() now has a verbosity parameter, so use it
-#
-# Revision 1.4  2004/05/02 22:54:43  ncq
-# - cleanup
-#
-# Revision 1.3  2004/04/30 09:10:57  ncq
-# - label needs to be str()ed in list.append()
-#
-# Revision 1.2  2004/03/10 12:56:01  ihaywood
-# fixed sudden loss of main.shadow
-# more work on referrals,
-#
-# Revision 1.1  2004/02/25 09:30:13  ncq
-# - moved here from python-common
-#
-# Revision 1.13  2004/01/12 13:10:27  ncq
-# - remove debugging code
-#
-# Revision 1.12  2004/01/06 10:02:47  ncq
-# - add _SQL2 match provider that operates on queries rather than tables/columns
-#
-# Revision 1.11  2003/12/29 16:28:04  uid66147
-# - I think we got the indentation level wrong when
-#   applying the extra condition default context
-#
-# Revision 1.10  2003/11/20 08:55:05  ncq
-# - some internal cleanup/renaming
-#
-# Revision 1.9  2003/11/20 02:16:03  sjtan
-#
-# make __context_val  in base class gmMatchProvider protected instead of class private, so subclasses can
-# access it.
-#
-# Revision 1.8  2003/11/20 01:37:05  sjtan
-#
-# syntax correction.
-#
-# Revision 1.7  2003/11/20 00:33:12  ncq
-# - improve comments on extra conditions in __find_matches()
-#
-# Revision 1.6  2003/11/19 23:18:37  ncq
-# - some cleanup
-#
