@@ -1,8 +1,8 @@
 """GNUmed allergy related widgets."""
 ############################################################################
 # $Source: /home/ncq/Projekte/cvs2git/vcs-mirror/gnumed/gnumed/client/wxpython/gmAllergyWidgets.py,v $
-# $Id: gmAllergyWidgets.py,v 1.35.4.1 2010-02-06 19:54:36 ncq Exp $
-__version__ = "$Revision: 1.35.4.1 $"
+# $Id: gmAllergyWidgets.py,v 1.36 2010-02-06 21:00:07 ncq Exp $
+__version__ = "$Revision: 1.36 $"
 __author__  = "R.Terry <rterry@gnumed.net>, H.Herb <hherb@gnumed.net>, K.Hilbert <Karsten.Hilbert@gmx.net>"
 __license__ = 'GPL (details at http://www.gnu.org)'
 
@@ -356,6 +356,8 @@ class cAllergyManagerDlg(wxgAllergyManagerDlg.wxgAllergyManagerDlg):
 	#--------------------------------------------------------
 	def _on_list_item_selected(self, evt):
 		allergy = self._LCTRL_allergies.get_selected_item_data(only_one=True)
+		if allergy is None:
+			return
 		self._PNL_edit_area.refresh(allergy=allergy)
 		self._BTN_delete.Enable(True)
 	#--------------------------------------------------------
@@ -565,8 +567,8 @@ if __name__ == "__main__":
 #		app.MainLoop()
 #======================================================================
 # $Log: gmAllergyWidgets.py,v $
-# Revision 1.35.4.1  2010-02-06 19:54:36  ncq
-# - fix exception on deleting allergy when none selected
+# Revision 1.36  2010-02-06 21:00:07  ncq
+# - do not fail deleting allergy if none selected
 #
 # Revision 1.35  2009/06/04 16:30:30  ncq
 # - use set active patient from pat search widgets
