@@ -7,9 +7,7 @@
 	and Karsten <Karsten.Hilbert@gmx.net>.
 """
 #================================================================
-# $Source: /home/ncq/Projekte/cvs2git/vcs-mirror/gnumed/gnumed/client/wxpython/gmEMRStructWidgets.py,v $
-# $Id: gmEMRStructWidgets.py,v 1.113.2.1 2010-02-04 12:01:22 ncq Exp $
-__version__ = "$Revision: 1.113.2.1 $"
+__version__ = "$Revision: 1.114 $"
 __author__ = "cfmoro1976@yahoo.es, karsten.hilbert@gmx.net"
 __license__ = "GPL"
 
@@ -431,6 +429,14 @@ class cHospitalStayEditAreaPnl(wxgHospitalStayEditAreaPnl.wxgHospitalStayEditAre
 				)
 				return False
 
+		if self._PRW_episode.GetValue().strip() == u'':
+			self._PRW_episode.display_as_valid(False)
+			wxps.Publisher().sendMessage (
+				topic = 'statustext',
+				data = {'msg': _('Must select an episode or enter a name for a new one. Cannot save hospital stay.'), 'beep': True}
+			)
+			return False
+
 		return True
 	#----------------------------------------------------------------
 	def _save_as_new(self):
@@ -545,7 +551,7 @@ def select_encounters(parent=None, patient=None, single_selection=True, encounte
 	return gmListWidgets.get_choices_from_list (
 		parent = parent,
 		msg = _('\nBelow find the relevant encounters of the patient.\n'),
-		caption = _('Showing encounters ...'),
+		caption = _('Encounters ...'),
 		columns = [_('Started'), _('Ended'), _('Type'), _('Reason for Encounter'), _('Assessment of Encounter'), _('Empty'), '#'],
 		can_return_empty = True,
 		single_selection = single_selection,
@@ -554,7 +560,10 @@ def select_encounters(parent=None, patient=None, single_selection=True, encounte
 	)
 #----------------------------------------------------------------
 def ask_for_encounter_continuation(msg=None, caption=None, encounter=None, parent=None):
-
+	"""This is used as the callback when the EMR detects that the
+	   patient was here rather recently and wants to ask the
+	   provider whether to continue the recent encounter.
+	"""
 	if parent is None:
 		parent = wx.GetApp().GetTopWindow()
 
@@ -1219,27 +1228,53 @@ limit 30"""
 		self.set_context('pat', self.__patient_id)
 		return True
 	#--------------------------------------------------------
-	def GetData(self, can_create=False, is_open=False):
-		if self.data is None:
-			if can_create:
-				epi_name = self.GetValue().strip()
-				if epi_name == u'':
-					gmDispatcher.send(signal = u'statustext',	msg = _('Cannot create episode without name.'),	beep = True)
-					_log.debug('cannot create episode without name')
-				else:
-					if self.use_current_patient:
-						pat = gmPerson.gmCurrentPatient()
-					else:
-						pat = gmPerson.cPatient(aPK_obj=self.__patient_id)
+	def GetData(self, can_create=False, as_instance=False, is_open=False):
 
-					emr = pat.get_emr()
-					epi = emr.add_episode(episode_name = epi_name, is_open = is_open)
-					if epi is None:
-						self.data = None
-					else:
-						self.data = epi['pk_episode']
+		self.__is_open = is_open
 
-		return gmPhraseWheel.cPhraseWheel.GetData(self)
+#		if self.data is None:
+#			if can_create:
+#				epi_name = self.GetValue().strip()
+#				if epi_name == u'':
+#					gmDispatcher.send(signal = u'statustext', msg = _('Cannot create episode without name.'), beep = True)
+#					_log.debug('cannot create episode without name')
+#				else:
+#					if self.use_current_patient:
+#						pat = gmPerson.gmCurrentPatient()
+#					else:
+#						pat = gmPerson.cPatient(aPK_obj = self.__patient_id)
+#
+#					emr = pat.get_emr()
+#					epi = emr.add_episode(episode_name = epi_name, is_open = is_open)
+#					if epi is None:
+#						self.data = None
+#					else:
+#						self.data = epi['pk_episode']
+
+		return gmPhraseWheel.cPhraseWheel.GetData(self, can_create = can_create, as_instance = as_instance)
+	#--------------------------------------------------------
+	def _create_data(self):
+
+		epi_name = self.GetValue().strip()
+		if epi_name == u'':
+			gmDispatcher.send(signal = u'statustext', msg = _('Cannot create episode without name.'), beep = True)
+			_log.debug('cannot create episode without name')
+			return
+
+		if self.use_current_patient:
+			pat = gmPerson.gmCurrentPatient()
+		else:
+			pat = gmPerson.cPatient(aPK_obj = self.__patient_id)
+
+		emr = pat.get_emr()
+		epi = emr.add_episode(episode_name = epi_name, is_open = self.__is_open)
+		if epi is None:
+			self.data = None
+		else:
+			self.data = epi['pk_episode']
+	#--------------------------------------------------------
+	def _data2instance(self):
+		return gmEMRStructItems.cEpisode(aPK_obj = self.data)
 	#--------------------------------------------------------
 	# internal API
 	#--------------------------------------------------------
@@ -1773,8 +1808,6 @@ limit 50""" % gmPerson.gmCurrentPatient().ID
 			return True
 
 		age = gmDateTime.str2interval(str_interval = str_age)
-		pat = gmPerson.gmCurrentPatient()
-		max_age = pydt.datetime.now(tz=pat['dob'].tzinfo) - pat['dob']
 
 		if age is None:
 			gmDispatcher.send(signal='statustext', msg=_('Cannot parse [%s] into valid interval.') % str_age)
@@ -1783,31 +1816,36 @@ limit 50""" % gmPerson.gmCurrentPatient().ID
 			wx.CallAfter(self._PRW_year_noted.SetText, u'', None, True)
 			return True
 
-		if age >= max_age:
-			gmDispatcher.send (
-				signal = 'statustext',
-				msg = _(
-					'Health issue cannot have been noted at age %s. Patient is only %s old.'
-				) % (age, pat.get_medical_age())
-			)
-			self._PRW_age_noted.SetBackgroundColour('pink')
-			self._PRW_age_noted.Refresh()
-			wx.CallAfter(self._PRW_year_noted.SetText, u'', None, True)
-			return True
+		pat = gmPerson.gmCurrentPatient()
+		if pat['dob'] is not None:
+			max_age = pydt.datetime.now(tz=pat['dob'].tzinfo) - pat['dob']
+
+			if age >= max_age:
+				gmDispatcher.send (
+					signal = 'statustext',
+					msg = _(
+						'Health issue cannot have been noted at age %s. Patient is only %s old.'
+					) % (age, pat.get_medical_age())
+				)
+				self._PRW_age_noted.SetBackgroundColour('pink')
+				self._PRW_age_noted.Refresh()
+				wx.CallAfter(self._PRW_year_noted.SetText, u'', None, True)
+				return True
 
 		self._PRW_age_noted.SetBackgroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW))
 		self._PRW_age_noted.Refresh()
 		self._PRW_age_noted.SetData(data=age)
 
-		fts = gmDateTime.cFuzzyTimestamp (
-			timestamp = pat['dob'] + age,
-			accuracy = gmDateTime.acc_months
-		)
-		wx.CallAfter(self._PRW_year_noted.SetText, str(fts), fts)
-		# if we do this we will *always* navigate there, regardless of TAB vs ALT-TAB
-		#wx.CallAfter(self._ChBOX_active.SetFocus)
-		# if we do the following instead it will take us to the save/update button ...
-		#wx.CallAfter(self.Navigate)
+		if pat['dob'] is not None:
+			fts = gmDateTime.cFuzzyTimestamp (
+				timestamp = pat['dob'] + age,
+				accuracy = gmDateTime.acc_months
+			)
+			wx.CallAfter(self._PRW_year_noted.SetText, str(fts), fts)
+			# if we do this we will *always* navigate there, regardless of TAB vs ALT-TAB
+			#wx.CallAfter(self._ChBOX_active.SetFocus)
+			# if we do the following instead it will take us to the save/update button ...
+			#wx.CallAfter(self.Navigate)
 
 		return True
 	#--------------------------------------------------------
@@ -1840,9 +1878,10 @@ limit 50""" % gmPerson.gmCurrentPatient().ID
 		self._PRW_year_noted.Refresh()
 
 		pat = gmPerson.gmCurrentPatient()
-		issue_age = year_noted - pat['dob']
-		str_age = gmDateTime.format_interval_medically(interval = issue_age)
-		wx.CallAfter(self._PRW_age_noted.SetText, str_age, issue_age)
+		if pat['dob'] is not None:
+			issue_age = year_noted - pat['dob']
+			str_age = gmDateTime.format_interval_medically(interval = issue_age)
+			wx.CallAfter(self._PRW_age_noted.SetText, str_age, issue_age)
 
 		return True
 	#--------------------------------------------------------
@@ -2024,418 +2063,3 @@ if __name__ == '__main__':
 		test_edit_procedure()
 
 #================================================================
-# $Log: gmEMRStructWidgets.py,v $
-# Revision 1.113.2.1  2010-02-04 12:01:22  ncq
-# - fix adding health issue
-#
-# Revision 1.113  2010/01/21 08:42:31  ncq
-# - adjust to Dx certainty mapper changes
-#
-# Revision 1.112  2009/12/03 17:48:15  ncq
-# - only create episode in GetData if there's a name available
-#
-# Revision 1.111  2009/12/01 10:50:44  ncq
-# - fix issue creation
-#
-# Revision 1.110  2009/11/15 01:05:44  ncq
-# - start-new-encounter
-# - enhance select-encounters
-#
-# Revision 1.109  2009/11/13 21:07:20  ncq
-# - fully implement procedure EA
-#
-# Revision 1.108  2009/11/06 15:17:46  ncq
-# - set better size on encounter EA
-#
-# Revision 1.107  2009/09/23 14:42:04  ncq
-# - implement procedure management
-# - implement promote-episode-to-issue
-#
-# Revision 1.106  2009/09/17 21:53:41  ncq
-# - start support for managing performed procedures
-#
-# Revision 1.105  2009/09/15 15:24:21  ncq
-# - start procedure EA
-# - phrasewheel for hospital stays
-#
-# Revision 1.104  2009/09/13 18:45:25  ncq
-# - no more get-active-encounter()
-#
-# Revision 1.103  2009/09/01 23:05:20  ncq
-# - improved tooltip for classification phrasewheel
-#
-# Revision 1.102  2009/09/01 22:29:09  ncq
-# - normalize issue/episode edit area handling, obsoleting their dialogs
-# - add edit_episode/edit_health_issue
-# - support diagnostic certainty
-#
-# Revision 1.101  2009/07/30 12:03:54  ncq
-# - fix editing age noted
-#
-# Revision 1.100  2009/07/16 09:52:15  ncq
-# - improved labelling
-#
-# Revision 1.99  2009/07/15 21:32:35  ncq
-# - add missing () thereby making changing enc type possible
-#
-# Revision 1.98  2009/07/09 16:46:20  ncq
-# - cleanup
-# - improved wording as per list
-#
-# Revision 1.97  2009/07/02 20:50:37  ncq
-# - use generic EA dlg 2
-#
-# Revision 1.96  2009/07/01 17:07:16  ncq
-# - in episode selector sort closed below unclosed episodes
-#
-# Revision 1.95  2009/06/29 15:05:52  ncq
-# - fix typo
-#
-# Revision 1.94  2009/06/11 12:37:25  ncq
-# - much simplified initial setup of list ctrls
-#
-# Revision 1.93  2009/06/04 16:30:30  ncq
-# - use set active patient from pat search widgets
-#
-# Revision 1.92  2009/05/13 13:12:21  ncq
-# - enable encounter editing right from the list
-#
-# Revision 1.91  2009/05/13 12:18:35  ncq
-# - streamline managing encounters
-#
-# Revision 1.90  2009/05/08 07:59:33  ncq
-# - cleanup, better display of encounter list
-#
-# Revision 1.89  2009/04/21 16:59:59  ncq
-# - edit area dlg now takes single_entry argument
-#
-# Revision 1.88  2009/04/19 22:26:47  ncq
-# - interval parsing moved
-#
-# Revision 1.87  2009/04/13 10:54:06  ncq
-# - show encounter list
-# - allow removing RFE/AOE from edit area
-#
-# Revision 1.86  2009/04/05 18:04:46  ncq
-# - support and use grouping
-#
-# Revision 1.85  2009/04/03 09:47:29  ncq
-# - hospital stay widgets
-#
-# Revision 1.84  2009/01/02 11:39:48  ncq
-# - support custom message/buttons in encounter edit area/dlg
-#
-# Revision 1.83  2008/12/09 23:28:51  ncq
-# - use description_gender
-#
-# Revision 1.82  2008/10/22 12:19:10  ncq
-# - rename pseudo episode on health issue creation to "inceptio notes"
-#
-# Revision 1.81  2008/10/12 16:15:17  ncq
-# - no more "foundational" health issue
-#
-# Revision 1.80  2008/09/02 19:01:12  ncq
-# - adjust to clin health_issue fk_patient drop and related changes
-#
-# Revision 1.79  2008/07/24 13:58:40  ncq
-# - manage encounter types
-#
-# Revision 1.78  2008/07/07 13:43:16  ncq
-# - current patient .connected
-#
-# Revision 1.77  2008/06/09 15:33:59  ncq
-# - improved episode selector SQL
-#
-# Revision 1.76  2008/05/13 14:11:53  ncq
-# - properly handle age=None in pHX ea
-#
-# Revision 1.75  2008/05/07 15:21:10  ncq
-# - move health issue EA behaviour close to Richard's specs
-#
-# Revision 1.74  2008/03/05 22:37:45  ncq
-# - new style logging
-# - new health issue adding
-#
-# Revision 1.73  2008/01/30 14:03:41  ncq
-# - use signal names directly
-# - switch to std lib logging
-#
-# Revision 1.72  2008/01/22 12:21:27  ncq
-# - better encounter editor
-#
-# Revision 1.71  2007/10/07 12:32:41  ncq
-# - workplace property now on gmSurgery.gmCurrentPractice() borg
-#
-# Revision 1.70  2007/08/29 22:08:57  ncq
-# - narrative widgets factored out
-#
-# Revision 1.69  2007/08/29 14:38:39  ncq
-# - improve encounter details dialog
-#
-# Revision 1.68  2007/08/15 09:19:32  ncq
-# - cleanup
-#
-# Revision 1.67  2007/08/13 22:00:48  ncq
-# - proper year format specs
-#
-# Revision 1.66  2007/08/13 11:07:41  ncq
-# - make episode descriptions phrasewheel actually *match*
-#   on input, IOW, add a where clause to the select ;-)
-#
-# Revision 1.65  2007/08/12 00:09:07  ncq
-# - no more gmSignals.py
-#
-# Revision 1.64  2007/07/13 12:20:48  ncq
-# - select_narrative_from_episodes(), related widgets, and test suite
-#
-# Revision 1.63  2007/06/10 10:02:53  ncq
-# - episode pk is pk_episode
-#
-# Revision 1.62  2007/05/18 13:28:57  ncq
-# - implement cMoveNarrativeDlg
-#
-# Revision 1.61  2007/05/14 13:11:24  ncq
-# - use statustext() signal
-#
-# Revision 1.60  2007/04/27 13:28:25  ncq
-# - use c2ButtonQuestionDlg
-#
-# Revision 1.59  2007/04/25 22:00:47  ncq
-# - use better question dialog for very recent encounter
-#
-# Revision 1.58  2007/04/13 15:58:00  ncq
-# - don't conflict open episode on itself
-#
-# Revision 1.57  2007/04/02 18:39:52  ncq
-# - gmFuzzyTimestamp -> gmDateTime
-#
-# Revision 1.56  2007/03/31 21:50:15  ncq
-# - cPatient now cIdentity child
-#
-# Revision 1.55  2007/03/18 14:05:31  ncq
-# - re-add lost 1.55
-#
-# Revision 1.55  2007/03/12 12:27:13  ncq
-# - convert some statustext calls to use signal handler
-#
-# Revision 1.54  2007/03/08 11:39:13  ncq
-# - cEpisodeSelectionPhraseWheel
-# 	- limit 30
-# 	- order by weight, description
-# 	- show both open and closed
-# 	- include health issue string
-# 	- test suite
-#
-# Revision 1.53  2007/02/22 17:41:13  ncq
-# - adjust to gmPerson changes
-#
-# Revision 1.52  2007/02/17 14:13:11  ncq
-# - gmPerson.gmCurrentProvider().workplace now property
-#
-# Revision 1.51  2007/02/16 12:53:19  ncq
-# - now that we have cPhraseWheel.suppress_text_update_smarts we
-#   can avoid infinite looping due to circular on_modified callbacks
-#
-# Revision 1.50  2007/02/09 14:59:39  ncq
-# - cleanup
-#
-# Revision 1.49  2007/02/06 13:43:40  ncq
-# - no more aDelay in __init__()
-#
-# Revision 1.48  2007/02/05 12:15:23  ncq
-# - no more aMatchProvider/selection_only in cPhraseWheel.__init__()
-#
-# Revision 1.47  2007/02/04 15:53:58  ncq
-# - use SetText()
-#
-# Revision 1.46  2007/01/15 20:22:09  ncq
-# - fix several bugs in move_episode_to_issue()
-#
-# Revision 1.45  2007/01/15 13:02:26  ncq
-# - completely revamped move_episode_to_issue() and use it
-#
-# Revision 1.44  2007/01/09 12:59:01  ncq
-# - datetime.timedelta needs int, not decimal, so make epi_ttl an int
-# - missing _ in front of CHBOX_closed
-# - save() needs to return True/False so dialog can close or not
-#
-# Revision 1.43  2007/01/04 23:29:02  ncq
-# - cEpisodeDescriptionPhraseWheel
-# - cEpisodeEditAreaPnl
-# - cEpisodeEditAreaDlg
-# - test suite enhancement
-#
-# Revision 1.42  2007/01/02 16:18:10  ncq
-# - health issue phrasewheel: clin.health_issue has fk_patient, not pk_patient
-#
-# Revision 1.41  2006/12/25 22:52:14  ncq
-# - encounter details editor
-#   - set patient name
-#   - valid_for_save(), save()
-#   - properly set started/ended
-#
-# Revision 1.40  2006/12/23 01:07:28  ncq
-# - fix encounter type phrasewheel query
-# - add encounter details edit area panel/dialog
-#   - still needs save() and friends fixed
-# - general cleanup/moving about of stuff
-# - fix move_episode_to_issue() logic
-# - start cleanup of test suite
-#
-# Revision 1.39  2006/11/28 20:44:36  ncq
-# - some rearrangement
-#
-# Revision 1.38  2006/11/27 23:15:01  ncq
-# - remove prints
-#
-# Revision 1.37  2006/11/27 23:05:49  ncq
-# - add commented out on_modified callbacks
-#
-# Revision 1.36  2006/11/27 12:40:20  ncq
-# - adapt to field name fixes from wxGlade
-#
-# Revision 1.35  2006/11/24 16:40:35  ncq
-# - age_noted can be NULL so handle set when refresh()ing health issue edit area
-#
-# Revision 1.34  2006/11/24 14:22:35  ncq
-# - cannot pass issue keyword to wx.Dialog child in cHealthIssueEditAreaDlg.__init__
-# - relabel buttons to save or update re clear/restore when adding/editing health issue
-# - EndModal needs argument
-#
-# Revision 1.33  2006/11/24 09:55:05  ncq
-# - cHealthIssueEditArea(Pnl/Dlg) closely following Richard's specs
-# - test code
-#
-# Revision 1.32  2006/11/15 00:40:07  ncq
-# - properly set up context for phrasewheels
-#
-# Revision 1.31  2006/10/31 17:21:16  ncq
-# - unicode()ify queries
-#
-# Revision 1.30  2006/10/24 13:22:40  ncq
-# - gmPG -> gmPG2
-# - no need for service name in cMatchProvider_SQL2()
-#
-# Revision 1.29  2006/09/03 11:30:28  ncq
-# - add move_episode_to_issue()
-#
-# Revision 1.28  2006/06/26 21:37:43  ncq
-# - cleanup
-#
-# Revision 1.27  2006/06/26 13:07:00  ncq
-# - fix issue selection phrasewheel SQL UNION
-# - improved variable naming
-# - track patient id in set_patient on issue/episode selection phrasewheel
-#   so GetData can create new issues/episodes if told to do so
-# - add cIssueSelectionDlg
-#
-# Revision 1.26  2006/06/23 21:32:11  ncq
-# - add cIssueSelectionPhrasewheel
-#
-# Revision 1.25  2006/05/31 09:46:20  ncq
-# - cleanup
-#
-# Revision 1.24  2006/05/28 15:40:51  ncq
-# - fix typo in variable
-#
-# Revision 1.23  2006/05/25 22:19:25  ncq
-# - add preconfigured episode selection/creation phrasewheel
-# - cleanup, fix unit test
-#
-# Revision 1.22  2006/05/04 09:49:20  ncq
-# - get_clinical_record() -> get_emr()
-# - adjust to changes in set_active_patient()
-# - need explicit set_active_patient() after ask_for_patient() if wanted
-#
-# Revision 1.21  2005/12/26 05:26:37  sjtan
-#
-# match schema
-#
-# Revision 1.20  2005/12/26 04:23:05  sjtan
-#
-# match schema changes.
-#
-# Revision 1.19  2005/12/06 14:24:15  ncq
-# - clin.clin_health_issue/episode -> clin.health_issue/episode
-#
-# Revision 1.18  2005/10/20 07:42:27  ncq
-# - somewhat improved edit area for issue
-#
-# Revision 1.17  2005/10/08 12:33:09  sjtan
-# tree can be updated now without refetching entire cache; done by passing emr object to create_xxxx methods and calling emr.update_cache(key,obj);refresh_historical_tree non-destructively checks for changes and removes removed nodes and adds them if cache mismatch.
-#
-# Revision 1.16  2005/10/04 19:24:53  sjtan
-# browser now remembers expansion state and select state between change of patients, between health issue rename, episode rename or encounter relinking. This helps when reviewing the record more than once in a day.
-#
-# Revision 1.15  2005/09/27 20:44:58  ncq
-# - wx.wx* -> wx.*
-#
-# Revision 1.14  2005/09/26 18:01:50  ncq
-# - use proper way to import wx26 vs wx2.4
-# - note: THIS WILL BREAK RUNNING THE CLIENT IN SOME PLACES
-# - time for fixup
-#
-# Revision 1.13  2005/09/24 09:17:28  ncq
-# - some wx2.6 compatibility fixes
-#
-# Revision 1.12  2005/08/06 16:50:51  ncq
-# - zero-size spacer seems pointless and besides it
-#   don't work like that in wx2.5
-#
-# Revision 1.11  2005/06/29 15:06:38  ncq
-# - defaults for edit area popup/editarea2 __init__
-#
-# Revision 1.10  2005/06/28 17:14:56  cfmoro
-# Auto size flag causes the text not being displayed
-#
-# Revision 1.9  2005/06/20 13:03:38  cfmoro
-# Relink encounter to another episode
-#
-# Revision 1.8  2005/06/10 23:22:43  ncq
-# - SQL2 match provider now requires query *list*
-#
-# Revision 1.7  2005/05/06 15:30:15  ncq
-# - attempt to properly set focus
-#
-# Revision 1.6  2005/04/25 08:30:59  ncq
-# - make past medical history proxy episodes closed by default
-#
-# Revision 1.5  2005/04/24 14:45:18  ncq
-# - cleanup, use generic edit area popup dialog
-# - "finalize" (as for 0.1) health issue edit area
-#
-# Revision 1.4  2005/04/20 22:09:54  ncq
-# - add edit area and popup dialog for health issue
-#
-# Revision 1.3  2005/03/14 14:36:31  ncq
-# - use simplified episode naming
-#
-# Revision 1.2  2005/01/31 18:51:08  ncq
-# - caching emr = patient.get_clinical_record() locally is unsafe
-#   because patient can change but emr will stay the same (it's a
-#   local "pointer", after all, and not a singleton)
-# - adding episodes actually works now
-#
-# Revision 1.1  2005/01/31 13:09:21  ncq
-# - this is OK to go in
-#
-# Revision 1.9  2005/01/31 13:06:02  ncq
-# - use gmPerson.ask_for_patient()
-#
-# Revision 1.8  2005/01/31 09:50:59  ncq
-# - gmPatient -> gmPerson
-#
-# Revision 1.7  2005/01/29 19:12:19  cfmoro
-# Episode creation on episode editor widget
-#
-# Revision 1.6  2005/01/29 18:01:20  ncq
-# - some cleanup
-# - actually create new episodes
-#
-# Revision 1.5  2005/01/28 18:05:56  cfmoro
-# Implemented episode picker and episode selector dialogs and widgets
-#
-# Revision 1.4  2005/01/24 16:57:38  ncq
-# - some cleanup here and there
-#
-#
