@@ -13,8 +13,8 @@ TODO:
 """
 #================================================================
 # $Source: /home/ncq/Projekte/cvs2git/vcs-mirror/gnumed/gnumed/client/wxpython/gmListWidgets.py,v $
-# $Id: gmListWidgets.py,v 1.35 2010-01-21 08:43:23 ncq Exp $
-__version__ = "$Revision: 1.35 $"
+# $Id: gmListWidgets.py,v 1.37 2010-02-06 21:37:26 ncq Exp $
+__version__ = "$Revision: 1.37 $"
 __author__ = "Karsten Hilbert <Karsten.Hilbert@gmx.net>"
 __license__ = "GPL"
 
@@ -34,7 +34,7 @@ from Gnumed.wxpython import gmGuiHelpers
 from Gnumed.wxGladeWidgets import wxgGenericListSelectorDlg, wxgGenericListManagerPnl
 
 #================================================================
-def get_choices_from_list(parent=None, msg=None, caption=None, choices=None, selections=None, columns=None, data=None, edit_callback=None, new_callback=None, delete_callback=None, refresh_callback=None, single_selection=False, can_return_empty=False):
+def get_choices_from_list(parent=None, msg=None, caption=None, choices=None, selections=None, columns=None, data=None, edit_callback=None, new_callback=None, delete_callback=None, refresh_callback=None, single_selection=False, can_return_empty=False, ignore_OK_button=False):
 	"""Let user select item(s) from a list.
 
 	- edit_callback: (item data)
@@ -56,12 +56,12 @@ def get_choices_from_list(parent=None, msg=None, caption=None, choices=None, sel
 	dlg.edit_callback = edit_callback
 	dlg.new_callback = new_callback
 	dlg.delete_callback = delete_callback
+	dlg.ignore_OK_button = ignore_OK_button
 	dlg.set_columns(columns = columns)
 
 	if refresh_callback is None:
 		dlg.set_string_items(items = choices)		# list ctrl will refresh anyway if possible
-
-	dlg.set_column_widths()
+		dlg.set_column_widths()
 
 	if data is not None:
 		dlg.set_data(data=data)						# can override data set if refresh_callback is not None
@@ -103,6 +103,8 @@ class cGenericListSelectorDlg(wxgGenericListSelectorDlg.wxgGenericListSelectorDl
 		self.edit_callback = None				# called when EDIT button pressed, data of topmost selected item passed in
 		self.delete_callback = None				# called when DELETE button pressed, data of topmost selected item passed in
 
+		self.ignore_OK_button = False			# by default do show/use the OK button
+
 		self.can_return_empty = False
 	#------------------------------------------------------------
 	def set_columns(self, columns=None):
@@ -128,18 +130,21 @@ class cGenericListSelectorDlg(wxgGenericListSelectorDlg.wxgGenericListSelectorDl
 	# event handlers
 	#------------------------------------------------------------
 	def _on_list_item_selected(self, event):
-		self._BTN_ok.Enable(True)
-		self._BTN_ok.SetDefault()
+		if not self.__ignore_OK_button:
+			self._BTN_ok.SetDefault()
+			self._BTN_ok.Enable(True)
+
 		if self.edit_callback is not None:
 			self._BTN_edit.Enable(True)
+
 		if self.delete_callback is not None:
 			self._BTN_delete.Enable(True)
 	#------------------------------------------------------------
 	def _on_list_item_deselected(self, event):
 		if self._LCTRL_items.get_selected_items(only_one=True) == -1:
 			if not self.can_return_empty:
-				self._BTN_ok.Enable(False)
 				self._BTN_cancel.SetDefault()
+				self._BTN_ok.Enable(False)
 			self._BTN_edit.Enable(False)
 			self._BTN_delete.Enable(False)
 	#------------------------------------------------------------
@@ -176,6 +181,17 @@ class cGenericListSelectorDlg(wxgGenericListSelectorDlg.wxgGenericListSelectorDl
 	#------------------------------------------------------------
 	# properties
 	#------------------------------------------------------------
+	def _set_ignore_OK_button(self, ignored):
+		self.__ignore_OK_button = ignored
+		if self.__ignore_OK_button:
+			self._BTN_ok.Enable(False)
+			self._BTN_ok.Hide()
+		else:
+			self._BTN_ok.Enable(True)
+			self._BTN_ok.Show()
+
+	ignore_OK_button = property(lambda x:x, _set_ignore_OK_button)
+	#------------------------------------------------------------
 	def _get_new_callback(self):
 		return self.__new_callback
 
@@ -239,13 +255,17 @@ class cGenericListSelectorDlg(wxgGenericListSelectorDlg.wxgGenericListSelectorDl
 	def _get_refresh_callback(self):
 		return self.__refresh_callback
 
+	def _set_refresh_callback_helper(self):
+		self.refresh_callback(lctrl = self._LCTRL_items)
+		self._LCTRL_items.set_column_widths()
+
 	def _set_refresh_callback(self, callback):
 		if callback is not None:
 			if not callable(callback):
 				raise ValueError('<refresh> callback is not a callable: %s' % callback)
 		self.__refresh_callback = callback
 		if callback is not None:
-			wx.CallAfter(self.__refresh_callback, lctrl = self._LCTRL_items)
+			wx.CallAfter(self._set_refresh_callback_helper)
 
 	refresh_callback = property(_get_refresh_callback, _set_refresh_callback)
 #================================================================
@@ -552,7 +572,13 @@ if __name__ == '__main__':
 
 #================================================================
 # $Log: gmListWidgets.py,v $
-# Revision 1.35  2010-01-21 08:43:23  ncq
+# Revision 1.37  2010-02-06 21:37:26  ncq
+# - support ignoring OK button
+#
+# Revision 1.36  2010/01/31 18:17:33  ncq
+# - make refresh callback setting smarter: set column widths after setting items
+#
+# Revision 1.35  2010/01/21 08:43:23  ncq
 # - somewhat support setting col widths within get-choice-from-list
 #
 # Revision 1.34  2009/11/28 18:30:07  ncq
