@@ -1,14 +1,11 @@
-"""GNUmed measurement widgets.
-"""
+"""GNUmed measurement widgets."""
 #================================================================
-# $Source: /home/ncq/Projekte/cvs2git/vcs-mirror/gnumed/gnumed/client/wxpython/gmMeasurementWidgets.py,v $
-# $Id: gmMeasurementWidgets.py,v 1.64.2.1 2010-02-01 14:56:32 ncq Exp $
-__version__ = "$Revision: 1.64.2.1 $"
+__version__ = "$Revision: 1.66 $"
 __author__ = "Karsten Hilbert <Karsten.Hilbert@gmx.net>"
 __license__ = "GPL"
 
 
-import sys, logging, datetime as pyDT, decimal, os
+import sys, logging, datetime as pyDT, decimal, os, webbrowser
 
 
 import wx, wx.grid, wx.lib.hyperlink
@@ -25,6 +22,7 @@ from Gnumed.wxGladeWidgets import wxgMeasurementEditAreaPnl
 
 _log = logging.getLogger('gm.ui')
 _log.info(__version__)
+
 #================================================================
 # LOINC related widgets
 #================================================================
@@ -73,6 +71,40 @@ def update_loinc_reference_data():
 #================================================================
 # convenience functions
 #================================================================
+def call_browser_on_measurement_type(measurement_type=None):
+
+	dbcfg = gmCfg.cCfgSQL()
+
+	url = dbcfg.get (
+		option = u'external.urls.measurements_search',
+		workplace = gmSurgery.gmCurrentPractice().active_workplace,
+		bias = 'user',
+		default = u"http://www.google.de/search?as_oq=%(search_term)s&num=10&as_sitesearch=laborlexikon.de"
+	)
+
+	base_url = dbcfg.get2 (
+		option = u'external.urls.measurements_encyclopedia',
+		workplace = gmSurgery.gmCurrentPractice().active_workplace,
+		bias = 'user',
+		default = u'http://www.laborlexikon.de'
+	)
+
+	if measurement_type is None:
+		url = base_url
+
+	measurement_type = measurement_type.strip()
+
+	if measurement_type == u'':
+		url = base_url
+
+	url = url % {'search_term': measurement_type}
+
+	webbrowser.open (
+		url = url,
+		new = False,
+		autoraise = True
+	)
+#----------------------------------------------------------------
 def edit_measurement(parent=None, measurement=None):
 	ea = cMeasurementEditAreaPnl(parent = parent, id = -1)
 	ea.data = measurement
@@ -545,6 +577,13 @@ class cMeasurementsGrid(wx.grid.Grid):
 			'ind': gmTools.coalesce(d['abnormality_indicator'], u'', u' (%s)'),
 			'pk_result': d['pk_test_result']
 		})
+		tmp = (u'%s%s' % (
+			gmTools.coalesce(d['name_test_org'], u''),
+			gmTools.coalesce(d['contact_test_org'], u'', u' (%s)'),
+		)).strip()
+		if tmp != u'':
+			tt += u' ' + _(u'Source: %s\n') % tmp
+		tt += u'\n'
 
 		# clinical evaluation
 		norm_eval = None
@@ -651,7 +690,6 @@ class cMeasurementsGrid(wx.grid.Grid):
 #			#-------------------------------------
 
 		# ranges
-		tt += u'\n'
 		tt += u' ' + _(u'Standard normal range: %(norm_min_max)s%(norm_range)s  \n') % ({
 			'norm_min_max': normal_min_max,
 			'norm_range': gmTools.coalesce (
@@ -664,7 +702,8 @@ class cMeasurementsGrid(wx.grid.Grid):
 				)
 			)
 		})
-		tt += u' ' + _(u'Reference group: %(ref_group)s\n') % ({'ref_group': gmTools.coalesce(d['norm_ref_group'], u'')})
+		if d['norm_ref_group'] is not None:
+			tt += u' ' + _(u'Reference group: %s\n') % d['norm_ref_group']
 		tt += u' ' + _(u'Clinical target range: %(clin_min_max)s%(clin_range)s  \n') % ({
 			'clin_min_max': clinical_min_max,
 			'clin_range': gmTools.coalesce (
@@ -679,12 +718,17 @@ class cMeasurementsGrid(wx.grid.Grid):
 		})
 
 		# metadata
-		tt += u' ' + _(u'Doc: %s\n') % _(u'\n Doc: ').join(gmTools.coalesce(d['comment'], u'').split(u'\n'))
-		tt += u' ' + _(u'Lab: %s\n') % _(u'\n Lab: ').join(gmTools.coalesce(d['note_test_org'], u'').split(u'\n'))
+		if d['comment'] is not None:
+			tt += u' ' + _(u'Doc: %s\n') % _(u'\n Doc: ').join(d['comment'].split(u'\n'))
+		if d['note_test_org'] is not None:
+			tt += u' ' + _(u'Lab: %s\n') % _(u'\n Lab: ').join(d['note_test_org'].split(u'\n'))
 		tt += u' ' + _(u'Episode: %s\n') % d['episode']
-		tt += u' ' + _(u'Issue: %s\n') % gmTools.coalesce(d['health_issue'], u'')
-		tt += u' ' + _(u'Material: %s\n') % gmTools.coalesce(d['material'], u'')
-		tt += u' ' + _(u'Details: %s\n') % gmTools.coalesce(d['material_detail'], u'')
+		if d['health_issue'] is not None:
+			tt += u' ' + _(u'Issue: %s\n') % d['health_issue']
+		if d['material'] is not None:
+			tt += u' ' + _(u'Material: %s\n') % d['material']
+		if d['material_detail'] is not None:
+			tt += u' ' + _(u'Details: %s\n') % d['material_detail']
 		tt += u'\n'
 
 		# review
@@ -696,11 +740,13 @@ class cMeasurementsGrid(wx.grid.Grid):
 			'sig_hand': gmTools.u_writing_hand,
 			'reviewed': review
 		})
-		tt += u' ' + _(u'Last reviewer: %(reviewer)s\n') % ({'reviewer': gmTools.bool2subst(d['review_by_you'], _('you'), gmTools.coalesce(d['last_reviewer'], u''))})
-		tt += u' ' + _(u' Technically abnormal: %(abnormal)s\n') % ({'abnormal': gmTools.bool2subst(d['is_technically_abnormal'], _('yes'), _('no'), u'')})
-		tt += u' ' + _(u' Clinically relevant: %(relevant)s\n') % ({'relevant': gmTools.bool2subst(d['is_clinically_relevant'], _('yes'), _('no'), u'')})
-		tt += u' ' + _(u' Comment: %s\n') % gmTools.coalesce(d['review_comment'], u'')
 		tt += u' ' + _(u'Responsible clinician: %s\n') % gmTools.bool2subst(d['you_are_responsible'], _('you'), d['responsible_reviewer'])
+		if d['reviewed']:
+			tt += u' ' + _(u'Last reviewer: %(reviewer)s\n') % ({'reviewer': gmTools.bool2subst(d['review_by_you'], _('you'), gmTools.coalesce(d['last_reviewer'], u'?'))})
+			tt += u' ' + _(u' Technically abnormal: %(abnormal)s\n') % ({'abnormal': gmTools.bool2subst(d['is_technically_abnormal'], _('yes'), _('no'), u'?')})
+			tt += u' ' + _(u' Clinically relevant: %(relevant)s\n') % ({'relevant': gmTools.bool2subst(d['is_clinically_relevant'], _('yes'), _('no'), u'?')})
+		if d['review_comment'] is not None:
+			tt += u' ' + _(u' Comment: %s\n') % d['review_comment'].strip()
 		tt += u'\n'
 
 		# type
@@ -710,8 +756,10 @@ class cMeasurementsGrid(wx.grid.Grid):
 			'abbrev_meta': gmTools.coalesce(d['abbrev_meta'], u''),
 			'pk_u_type': d['pk_meta_test_type']
 		})
-		tt += u' ' + _(u'Type comment: %s\n') % _(u'\n Type comment:').join(gmTools.coalesce(d['comment_tt'], u'').split(u'\n'))
-		tt += u' ' + _(u'Group comment: %s\n') % _(u'\n Group comment: ').join(gmTools.coalesce(d['comment_meta'], u'').split(u'\n'))
+		if d['comment_tt'] is not None:
+			tt += u' ' + _(u'Type comment: %s\n') % _(u'\n Type comment:').join(d['comment_tt'].split(u'\n'))
+		if d['comment_meta'] is not None:
+			tt += u' ' + _(u'Group comment: %s\n') % _(u'\n Group comment: ').join(d['comment_meta'].split(u'\n'))
 		tt += u'\n'
 
 		tt += _(u'Revisions: %(row_ver)s, last %(mod_when)s by %(mod_by)s.') % ({
@@ -1359,6 +1407,23 @@ class cMeasurementEditAreaPnl(wxgMeasurementEditAreaPnl.wxgMeasurementEditAreaPn
 		self._CHBOX_abnormal.Enable(self._CHBOX_review.GetValue())
 		self._CHBOX_relevant.Enable(self._CHBOX_review.GetValue())
 		self._TCTRL_review_comment.Enable(self._CHBOX_review.GetValue())
+	#--------------------------------------------------------
+	def _on_test_info_button_pressed(self, event):
+
+		pk = self._PRW_test.GetData()
+		if pk is not None:
+			tt = gmPathLab.cMeasurementType(aPK_obj = pk)
+			search_term = u'%s %s %s' % (
+				tt['name'],
+				tt['abbrev'],
+				gmTools.coalesce(tt['loinc'], u'')
+			)
+		else:
+			search_term = self._PRW_test.GetValue()
+
+		search_term = search_term.replace(' ', u'+')
+
+		call_browser_on_measurement_type(measurement_type = search_term)
 #================================================================
 # measurement type handling
 #================================================================
@@ -1513,6 +1578,40 @@ limit 50""" % {'in_house': _('in house lab')}
 		self.SetToolTipString(_('Select the type of measurement.'))
 		self.selection_only = False
 #----------------------------------------------------------------
+class cMeasurementOrgPhraseWheel(gmPhraseWheel.cPhraseWheel):
+
+	def __init__(self, *args, **kwargs):
+
+		query = u"""
+select distinct on (internal_name)
+	pk,
+	internal_name
+from clin.test_org
+where
+	internal_name %(fragment_condition)s
+order by internal_name
+limit 50"""
+		mp = gmMatchProvider.cMatchProvider_SQL2(queries=query)
+		mp.setThresholds(1, 2, 4)
+		#mp.word_separators = '[ \t:@]+'
+		gmPhraseWheel.cPhraseWheel.__init__(self, *args, **kwargs)
+		self.matcher = mp
+		self.SetToolTipString(_('The name of the path lab/diagnostic organisation.'))
+		self.selection_only = False
+	#------------------------------------------------------------
+	def _create_data(self):
+		if self.data is not None:
+			_log.debug('data already set, not creating')
+			return
+
+		if self.GetValue().strip() == u'':
+			_log.debug('cannot create new lab, missing name')
+			return
+
+		lab = gmPathLab.create_test_org(name = self.GetValue().strip())
+		self.SetText(value = lab['internal_name'], data = lab['pk'])
+		return
+#----------------------------------------------------------------
 from Gnumed.wxGladeWidgets import wxgMeasurementTypeEAPnl
 
 class cMeasurementTypeEAPnl(wxgMeasurementTypeEAPnl.wxgMeasurementTypeEAPnl, gmEditArea.cGenericEditAreaMixin):
@@ -1645,20 +1744,20 @@ limit 50"""
 		self._PRW_loinc.selection_only = False
 		self._PRW_loinc.add_callback_on_lose_focus(callback = self._on_loinc_lost_focus)
 
-		# test org
-		query = u"""
-select distinct on (internal_name)
-	pk,
-	internal_name
-from clin.test_org
-where
-	internal_name %(fragment_condition)s
-order by internal_name
-limit 50"""
-		mp = gmMatchProvider.cMatchProvider_SQL2(queries=query)
-		mp.setThresholds(1, 2, 4)
-		self._PRW_test_org.matcher = mp
-		self._PRW_test_org.selection_only = False
+#		# test org
+#		query = u"""
+#select distinct on (internal_name)
+#	pk,
+#	internal_name
+#from clin.test_org
+#where
+#	internal_name %(fragment_condition)s
+#order by internal_name
+#limit 50"""
+#		mp = gmMatchProvider.cMatchProvider_SQL2(queries=query)
+#		mp.setThresholds(1, 2, 4)
+#		self._PRW_test_org.matcher = mp
+#		self._PRW_test_org.selection_only = False
 	#----------------------------------------------------------------
 	def _on_loinc_lost_focus(self):
 		loinc = self._PRW_loinc.GetData()
@@ -1842,6 +1941,136 @@ limit 25"""
 		self.SetToolTipString(_('Select an indicator for the level of abnormality.'))
 		self.selection_only = False
 #================================================================
+# measurement org widgets / functions
+#----------------------------------------------------------------
+def edit_measurement_org(parent=None, org=None):
+	ea = cMeasurementOrgEAPnl(parent = parent, id = -1)
+	ea.data = org
+	ea.mode = gmTools.coalesce(org, 'new', 'edit')
+	dlg = gmEditArea.cGenericEditAreaDlg2(parent = parent, id = -1, edit_area = ea)
+	dlg.SetTitle(gmTools.coalesce(org, _('Adding new diagnostic org'), _('Editing diagnostic org')))
+	if dlg.ShowModal() == wx.ID_OK:
+		dlg.Destroy()
+		return True
+	dlg.Destroy()
+	return False
+#----------------------------------------------------------------
+def manage_measurement_orgs(parent=None):
+
+	if parent is None:
+		parent = wx.GetApp().GetTopWindow()
+
+	#------------------------------------------------------------
+	def edit(org=None):
+		return edit_measurement_org(parent = parent, org = org)
+	#------------------------------------------------------------
+	def refresh(lctrl):
+		orgs = gmPathLab.get_test_orgs()
+		lctrl.set_string_items ([
+			(o['internal_name'], gmTools.coalesce(o['contact'], u''), gmTools.coalesce(o['comment']), o['pk'])
+			for o in orgs
+		])
+		lctrl.set_data(orgs)
+	#------------------------------------------------------------
+	def delete(measurement_type):
+		if measurement_type.in_use:
+			gmDispatcher.send (
+				signal = 'statustext',
+				beep = True,
+				msg = _('Cannot delete measurement type [%s (%s)] because it is in use.') % (measurement_type['name'], measurement_type['abbrev'])
+			)
+			return False
+		gmPathLab.delete_measurement_type(measurement_type = measurement_type['pk_test_type'])
+		return True
+	#------------------------------------------------------------
+	gmListWidgets.get_choices_from_list (
+		parent = parent,
+		msg = _('\nThese are the diagnostic orgs (path labs etc) currently defined in GNUmed.\n\n'),
+		caption = _('Showing diagnostic orgs.'),
+		columns = [_('Name'), _('Contact'), _('Comment'), u'#'],
+		single_selection = True,
+		refresh_callback = refresh,
+		edit_callback = edit,
+		new_callback = edit
+#		,delete_callback = delete
+	)
+
+
+#----------------------------------------------------------------
+from Gnumed.wxGladeWidgets import wxgMeasurementOrgEAPnl
+
+class cMeasurementOrgEAPnl(wxgMeasurementOrgEAPnl.wxgMeasurementOrgEAPnl, gmEditArea.cGenericEditAreaMixin):
+
+	def __init__(self, *args, **kwargs):
+
+		try:
+			data = kwargs['org']
+			del kwargs['org']
+		except KeyError:
+			data = None
+
+		wxgMeasurementOrgEAPnl.wxgMeasurementOrgEAPnl.__init__(self, *args, **kwargs)
+		gmEditArea.cGenericEditAreaMixin.__init__(self)
+
+		# Code using this mixin should set mode and data
+		# after instantiating the class:
+		self.mode = 'new'
+		self.data = data
+		if data is not None:
+			self.mode = 'edit'
+
+		#self.__init_ui()
+	#----------------------------------------------------------------
+#	def __init_ui(self):
+#		# adjust phrasewheels etc
+	#----------------------------------------------------------------
+	# generic Edit Area mixin API
+	#----------------------------------------------------------------
+	def _valid_for_save(self):
+		has_errors = False
+		if self._PRW_name.GetValue().strip() == u'':
+			has_errors = True
+			self._PRW_name.display_as_valid(valid = False)
+		else:
+			self._PRW_name.display_as_valid(valid = True)
+
+		return (not has_errors)
+	#----------------------------------------------------------------
+	def _save_as_new(self):
+		# save the data as a new instance
+		data = self._PRW_name.GetData(can_create = True)
+
+		data['contact'] = self._TCTRL_contact.GetValue().strip()
+		data['comment'] = self._TCTRL_comment.GetValue().strip()
+		data.save()
+
+		# must be done very late or else the property access
+		# will refresh the display such that later field
+		# access will return empty values
+		self.data = data
+
+		return True
+	#----------------------------------------------------------------
+	def _save_as_update(self):
+		self.data['internal_name'] = self._PRW_name.GetValue().strip()
+		self.data['contact'] = self._TCTRL_contact.GetValue().strip()
+		self.data['comment'] = self._TCTRL_comment.GetValue().strip()
+		self.data.save()
+		return True
+	#----------------------------------------------------------------
+	def _refresh_as_new(self):
+		self._PRW_name.SetText(value = u'', data = None)
+		self._TCTRL_contact.SetValue(u'')
+		self._TCTRL_comment.SetValue(u'')
+	#----------------------------------------------------------------
+	def _refresh_from_existing(self):
+		self._PRW_name.SetText(value = self.data['internal_name'], data = self.data['pk'])
+		self._TCTRL_contact.SetValue(gmTools.coalesce(self.data['contact'], u''))
+		self._TCTRL_comment.SetValue(gmTools.coalesce(self.data['comment'], u''))
+	#----------------------------------------------------------------
+	def _refresh_as_new_from_existing(self):
+		self._refresh_as_new()
+#================================================================
 def manage_meta_test_types(parent=None):
 
 	if parent is None:
@@ -1920,255 +2149,4 @@ if __name__ == '__main__':
 		#test_primary_care_vitals_pnl()
 
 #================================================================
-# $Log: gmMeasurementWidgets.py,v $
-# Revision 1.64.2.1  2010-02-01 14:56:32  ncq
-# - fix faulty access to abnormality indicator PRW content
-#
-# Revision 1.64  2009/12/21 15:12:29  ncq
-# - cleanup
-# - fix typo
-# - missing return
-#
-# Revision 1.63  2009/12/03 17:50:20  ncq
-# - row label tooltips
-#
-# Revision 1.62  2009/12/01 21:54:04  ncq
-# - cleanup
-#
-# Revision 1.61  2009/10/28 16:42:53  ncq
-# - make grid draggable
-#
-# Revision 1.60  2009/09/17 21:54:55  ncq
-# - properly access test type pk
-# - check for use before deleting test type
-#
-# Revision 1.59  2009/09/01 22:33:25  ncq
-# - order test types in list
-#
-# Revision 1.58  2009/08/24 20:11:27  ncq
-# - bump db version
-# - fix tag creation
-# - provider inbox:
-# 	enable filter-to-active-patient,
-# 	listen to new signal,
-# 	use cInboxMessage class
-# - properly constrain LOINC phrasewheel SQL
-# - include v12 scripts in release
-# - install arriba jar to /usr/local/bin/
-# - check for table existence in audit schema generator
-# - include dem.message inbox with additional generic signals
-#
-# Revision 1.57  2009/08/11 10:49:23  ncq
-# - cleanup
-# - remove LOINC files after import
-# - row labels now "abbrev (desc)", again
-# - Encyclopedia -> Reference
-# - improved LOINC matcher and use loinc to set loinc info
-#
-# Revision 1.56  2009/08/08 12:18:12  ncq
-# - setup phrasewheels in measurement type EA
-#
-# Revision 1.55  2009/08/03 20:50:48  ncq
-# - properly support adding/editing measurement type
-#
-# Revision 1.54  2009/07/20 20:33:35  ncq
-# - start implementing test type management
-#
-# Revision 1.53  2009/07/15 12:22:46  ncq
-# - fix incomplete validity check for new-result problem
-#
-# Revision 1.52  2009/07/06 17:15:45  ncq
-# - row labels only test name until proper support for abbrev is there
-# - improved formatting of test result for display in cell
-# - only remind of display being most-recent only if cell actually is multi-result in cell tooltip
-# - use successful-save message on EA
-# - safer refresh after save-and-next-value
-#
-# Revision 1.51  2009/07/02 20:54:05  ncq
-# - fix bug where second patient didn't show measurements on patient change
-#
-# Revision 1.50  2009/06/22 09:26:49  ncq
-# - people didn't like the bandwidth calculation
-#
-# Revision 1.49  2009/06/20 22:38:05  ncq
-# - factor out cell tooltip creation and only do it on mouse over
-#
-# Revision 1.48  2009/06/11 12:37:25  ncq
-# - much simplified initial setup of list ctrls
-#
-# Revision 1.47  2009/06/04 16:19:00  ncq
-# - re-adjust to test table changes
-# - update loinc
-# - adjust to list widget changes (refresh)
-#
-# Revision 1.47  2009/05/28 10:53:40  ncq
-# - adjust to test tables changes
-#
-# Revision 1.46  2009/05/24 16:29:14  ncq
-# - support (meta) test types
-#
-# Revision 1.45  2009/04/24 12:05:20  ncq
-# - properly display lab link in grid corner
-#
-# Revision 1.44  2009/04/21 17:01:12  ncq
-# - try various other things to try to center the lab link
-#
-# Revision 1.43  2009/04/19 22:28:23  ncq
-# - put hyperlink in upper left corner of lab grid
-#
-# Revision 1.42  2009/04/14 18:35:27  ncq
-# - HCI screening revealed test types scroll off when
-#   moving horizontall so fix that
-#
-# Revision 1.41  2009/04/03 09:50:21  ncq
-# - comment
-#
-# Revision 1.40  2009/03/18 14:30:47  ncq
-# - improved result tooltip
-#
-# Revision 1.39  2009/03/01 18:15:55  ncq
-# - lots of missing u'', decode strftime results
-# - adjust word separators in test type match provider
-#
-# Revision 1.38  2009/02/20 15:43:21  ncq
-# - u''ify
-#
-# Revision 1.37  2009/02/17 17:47:31  ncq
-# - comment out primary care vitals
-#
-# Revision 1.36  2009/02/12 16:23:39  ncq
-# - start work on primary care vitals input
-#
-# Revision 1.35  2009/01/28 11:27:56  ncq
-# - slightly better naming and comments
-#
-# Revision 1.34  2009/01/02 11:40:27  ncq
-# - properly check for numericity of value/range input
-#
-# Revision 1.33  2008/10/22 12:21:57  ncq
-# - use %x in strftime where appropriate
-#
-# Revision 1.32  2008/08/31 18:21:54  ncq
-# - work around Windows' inability to do nothing when
-#   there's nothing to do
-#
-# Revision 1.31  2008/08/31 18:04:30  ncq
-# - properly handle cell data now being list in select_cells()
-#
-# Revision 1.30  2008/08/31 17:13:50  ncq
-# - don't crash on double-clicking empty test results cell
-#
-# Revision 1.29  2008/08/31 17:04:17  ncq
-# - need to cast val_normal/target_min/max to unicode before display
-#
-# Revision 1.28  2008/08/15 15:57:10  ncq
-# - indicate data revisions in tooltip
-#
-# Revision 1.27  2008/08/08 13:31:58  ncq
-# - better results layout
-#
-# Revision 1.26  2008/08/05 16:21:30  ncq
-# - support multiple values per cell
-#
-# Revision 1.25  2008/07/17 21:41:36  ncq
-# - cleanup
-#
-# Revision 1.24  2008/07/14 13:47:36  ncq
-# - explicitely set focus after refresh per user request
-#
-# Revision 1.23  2008/07/13 16:13:33  ncq
-# - add_new_measurement -> edit_measurement
-# - use cGenericEditAreaMixin on results edit area
-# - invoked results edit area via double-click on result in grid
-#
-# Revision 1.22  2008/07/07 13:43:17  ncq
-# - current patient .connected
-#
-# Revision 1.21  2008/06/24 14:00:09  ncq
-# - action button popup menu
-# - handle result deletion
-#
-# Revision 1.20  2008/06/23 21:50:26  ncq
-# - create test types on the fly
-#
-# Revision 1.19  2008/06/22 17:32:39  ncq
-# - implement refresh on measurement ea so "Next" will work in dialog
-#
-# Revision 1.18  2008/06/19 15:26:09  ncq
-# - finish saving test result from edit area
-# - fix a few oversights in the result tooltip
-#
-# Revision 1.17  2008/06/18 15:49:22  ncq
-# - improve save validity check on edit area
-#
-# Revision 1.16  2008/06/16 15:03:20  ncq
-# - first cut at saving test results
-#
-# Revision 1.15  2008/06/15 20:43:31  ncq
-# - add test result indicator phrasewheel
-#
-# Revision 1.14  2008/06/09 15:36:04  ncq
-# - reordered for clarity
-# - add_new_measurement
-# - edit area start
-# - phrasewheels
-#
-# Revision 1.13  2008/05/14 15:01:43  ncq
-# - remove spurious evt argument in _on_pre_patient_selection()
-#
-# Revision 1.12  2008/04/26 21:40:58  ncq
-# - eventually support selecting certain ranges of cells
-#
-# Revision 1.11  2008/04/26 10:05:32  ncq
-# - in review dialog when user is already responsible
-#   disable make_me_responsible checkbox
-#
-# Revision 1.10  2008/04/22 21:18:49  ncq
-# - implement signing
-# - improved tooltip
-# - properly clear grid when active patient changes
-#
-# Revision 1.9  2008/04/16 20:39:39  ncq
-# - working versions of the wxGlade code and use it, too
-# - show client version in login dialog
-#
-# Revision 1.8  2008/04/11 23:12:23  ncq
-# - improve docs
-#
-# Revision 1.7  2008/04/04 13:09:45  ncq
-# - use +/- as abnormality indicator where not available
-# - more complete calculation of "more result data available"
-#
-# Revision 1.6  2008/04/02 10:48:33  ncq
-# - cleanup, review -> sign
-# - support test_count in review widget
-# - better results formatting as per list discussion
-#
-# Revision 1.5  2008/03/29 16:19:57  ncq
-# - review_current_selection()
-# - get_selected_cells()
-# - bold test names
-# - display abnormality indicator and clinical relevance
-# - improve tooltip
-# - cMeasurementsReviewDialog()
-# - listen to test result database changes
-#
-# Revision 1.4  2008/03/25 19:36:30  ncq
-# - fix imports
-# - better docs
-# - str() wants non-u''
-# - cMeasurementsPnl()
-#
-# Revision 1.3  2008/03/20 15:31:40  ncq
-# - improve cell tooltips with review status and issue/episode information
-# - start row labels tooltips
-#
-# Revision 1.2  2008/03/17 14:55:41  ncq
-# - add lots of TODOs
-# - better layout
-# - set grid cell tooltips
-#
-# Revision 1.1  2008/03/16 11:57:47  ncq
-# - first iteration
-#
-#
+
