@@ -1,11 +1,9 @@
 """GNUmed narrative handling widgets."""
 #================================================================
-# $Source: /home/ncq/Projekte/cvs2git/vcs-mirror/gnumed/gnumed/client/wxpython/gmNarrativeWidgets.py,v $
-# $Id: gmNarrativeWidgets.py,v 1.45 2010-01-11 19:51:09 ncq Exp $
-__version__ = "$Revision: 1.45 $"
+__version__ = "$Revision: 1.46 $"
 __author__ = "Karsten Hilbert <Karsten.Hilbert@gmx.net>"
 
-import sys, logging, os, os.path, time, re as regex
+import sys, logging, os, os.path, time, re as regex, shutil
 
 
 import wx
@@ -14,11 +12,14 @@ import wx.lib.expando as wxexpando
 
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
-from Gnumed.pycommon import gmI18N, gmDispatcher, gmTools, gmDateTime, gmPG2, gmCfg
+from Gnumed.pycommon import gmI18N, gmDispatcher, gmTools, gmDateTime
+from Gnumed.pycommon import gmShellAPI, gmPG2, gmCfg, gmMatchProvider
 from Gnumed.business import gmPerson, gmEMRStructItems, gmClinNarrative, gmSurgery
+from Gnumed.business import gmForms, gmDocuments
+from Gnumed.wxpython import gmListWidgets, gmEMRStructWidgets, gmRegetMixin
+from Gnumed.wxpython import gmPhraseWheel, gmGuiHelpers, gmPatSearchWidgets
+from Gnumed.wxpython import gmCfgWidgets, gmDocumentWidgets
 from Gnumed.exporters import gmPatientExporter
-from Gnumed.wxpython import gmListWidgets, gmEMRStructWidgets, gmRegetMixin, gmGuiHelpers, gmPatSearchWidgets
-from Gnumed.wxGladeWidgets import wxgMoveNarrativeDlg, wxgSoapNoteExpandoEditAreaPnl
 
 
 _log = logging.getLogger('gm.ui')
@@ -147,11 +148,8 @@ def manage_progress_notes(parent=None, encounters=None, episodes=None, patient=N
 			parent,
 			-1,
 			title = _('Editing progress note'),
-			msg = _(
-				'This is the original progress note:\n'
-				'\n'
-				' %s'
-			) % item.format(left_margin = u' ', fancy = True),
+			msg = _('This is the original progress note:'),
+			data = item.format(left_margin = u' ', fancy = True),
 			text = item['narrative']
 		)
 		decision = dlg.ShowModal()
@@ -197,7 +195,8 @@ def manage_progress_notes(parent=None, encounters=None, episodes=None, patient=N
 		can_return_empty = False,
 		edit_callback = edit,
 		delete_callback = delete,
-		refresh_callback = refresh
+		refresh_callback = refresh,
+		ignore_OK_button = True
 	)
 #------------------------------------------------------------
 def search_narrative_across_emrs(parent=None):
@@ -516,6 +515,8 @@ class cNarrativeListSelectorDlg(gmListWidgets.cGenericListSelectorDlg):
 		self._LCTRL_items.set_column_widths()
 		self._LCTRL_items.set_data(data = narrative)
 #------------------------------------------------------------
+from Gnumed.wxGladeWidgets import wxgMoveNarrativeDlg
+
 class cMoveNarrativeDlg(wxgMoveNarrativeDlg.wxgMoveNarrativeDlg):
 
 	def __init__(self, *args, **kwargs):
@@ -579,6 +580,7 @@ class cSoapPluginPnl(wxgSoapPluginPnl.wxgSoapPluginPnl, gmRegetMixin.cRegetOnPai
 	- previous notes
 	- notebook with progress note editors
 	- encounter details fields
+	- visual soap area
 
 	Listens to patient change signals, thus acts on the current patient.
 	"""
@@ -630,6 +632,7 @@ class cSoapPluginPnl(wxgSoapPluginPnl.wxgSoapPluginPnl, gmRegetMixin.cRegetOnPai
 		self._splitter_main.SetSashGravity(0.5)
 		self._splitter_left.SetSashGravity(0.5)
 		self._splitter_right.SetSashGravity(1.0)
+		self._splitter_soap.SetSashGravity(0.75)
 
 		splitter_size = self._splitter_main.GetSizeTuple()[0]
 		self._splitter_main.SetSashPosition(splitter_size * 3 / 10, True)
@@ -640,6 +643,9 @@ class cSoapPluginPnl(wxgSoapPluginPnl.wxgSoapPluginPnl, gmRegetMixin.cRegetOnPai
 		splitter_size = self._splitter_right.GetSizeTuple()[1]
 		self._splitter_right.SetSashPosition(splitter_size * 15 / 20, True)
 
+		splitter_size = self._splitter_soap.GetSizeTuple()[0]
+		self._splitter_soap.SetSashPosition(splitter_size * 3 / 4, True)
+
 		self._NB_soap_editors.DeleteAllPages()
 	#--------------------------------------------------------
 	def __reset_ui_content(self):
@@ -647,15 +653,24 @@ class cSoapPluginPnl(wxgSoapPluginPnl.wxgSoapPluginPnl, gmRegetMixin.cRegetOnPai
 		Clear all information from input panel
 		"""
 		self._LCTRL_active_problems.set_string_items()
-		self._lbl_hints.SetLabel(u'')
+
 		self._TCTRL_recent_notes.SetValue(u'')
-		self._NB_soap_editors.DeleteAllPages()
-		self._NB_soap_editors.add_editor()
+
 		self._PRW_encounter_type.SetText(suppress_smarts = True)
 		self._PRW_encounter_start.SetText(suppress_smarts = True)
 		self._PRW_encounter_end.SetText(suppress_smarts = True)
 		self._TCTRL_rfe.SetValue(u'')
 		self._TCTRL_aoe.SetValue(u'')
+
+		self._NB_soap_editors.DeleteAllPages()
+		self._NB_soap_editors.add_editor()
+
+		self._PNL_visual_soap.clear()
+
+		self._lbl_hints.SetLabel(u'')
+	#--------------------------------------------------------
+	def __refresh_visual_soaps(self):
+		self._PNL_visual_soap.refresh()
 	#--------------------------------------------------------
 	def __refresh_problem_list(self):
 		"""Update health problems list.
@@ -841,7 +856,8 @@ class cSoapPluginPnl(wxgSoapPluginPnl.wxgSoapPluginPnl, gmRegetMixin.cRegetOnPai
 			'pk_type': self._PRW_encounter_type.GetData(),
 			'reason_for_encounter': gmTools.none_if(self._TCTRL_rfe.GetValue().strip(), u''),
 			'assessment_of_encounter': gmTools.none_if(self._TCTRL_aoe.GetValue().strip(), u''),
-			'pk_location': enc['pk_location']
+			'pk_location': enc['pk_location'],
+			'pk_patient': enc['pk_patient']
 		}
 
 		if self._PRW_encounter_start.GetData() is None:
@@ -854,7 +870,7 @@ class cSoapPluginPnl(wxgSoapPluginPnl.wxgSoapPluginPnl, gmRegetMixin.cRegetOnPai
 		else:
 			data['last_affirmed'] = self._PRW_encounter_end.GetData().get_pydt()
 
-		return enc.same_payload(another_object = data)
+		return not enc.same_payload(another_object = data)
 	#--------------------------------------------------------
 	def __encounter_valid_for_save(self):
 
@@ -887,8 +903,9 @@ class cSoapPluginPnl(wxgSoapPluginPnl.wxgSoapPluginPnl, gmRegetMixin.cRegetOnPai
 		gmDispatcher.connect(signal = u'post_patient_selection', receiver = self._on_post_patient_selection)
 		gmDispatcher.connect(signal = u'episode_mod_db', receiver = self._on_episode_issue_mod_db)
 		gmDispatcher.connect(signal = u'health_issue_mod_db', receiver = self._on_episode_issue_mod_db)
+		gmDispatcher.connect(signal = u'doc_mod_db', receiver = self._on_doc_mod_db)
 		gmDispatcher.connect(signal = u'current_encounter_modified', receiver = self._on_current_encounter_modified)
-		gmDispatcher.connect(signal = u'current_encounter_switched', receiver = self._on_current_encounter_modified)
+		gmDispatcher.connect(signal = u'current_encounter_switched', receiver = self._on_current_encounter_switched)
 
 		# synchronous signals
 		self.__pat.register_pre_selection_callback(callback = self._pre_selection_callback)
@@ -941,12 +958,22 @@ class cSoapPluginPnl(wxgSoapPluginPnl.wxgSoapPluginPnl, gmRegetMixin.cRegetOnPai
 	def _on_post_patient_selection(self):
 		wx.CallAfter(self._schedule_data_reget)
 	#--------------------------------------------------------
+	def _on_doc_mod_db(self):
+		wx.CallAfter(self.__refresh_visual_soaps)
+	#--------------------------------------------------------
 	def _on_episode_issue_mod_db(self):
 		wx.CallAfter(self._schedule_data_reget)
 	#--------------------------------------------------------
 	def _on_current_encounter_modified(self):
 		wx.CallAfter(self.__refresh_encounter)
 	#--------------------------------------------------------
+	def _on_current_encounter_switched(self):
+		wx.CallAfter(self.__on_current_encounter_switched)
+	#--------------------------------------------------------
+	def __on_current_encounter_switched(self):
+		self.__refresh_encounter()
+		self.__refresh_visual_soaps()
+	#--------------------------------------------------------
 	def _on_problem_focused(self, event):
 		"""Show related note at the bottom."""
 		pass
@@ -1052,6 +1079,7 @@ class cSoapPluginPnl(wxgSoapPluginPnl.wxgSoapPluginPnl, gmRegetMixin.cRegetOnPai
 	def _populate_with_data(self):
 		self.__refresh_problem_list()
 		self.__refresh_encounter()
+		self.__refresh_visual_soaps()
 		return True
 #============================================================
 class cSoapNoteInputNotebook(wx.Notebook):
@@ -1195,7 +1223,10 @@ class cSoapNoteInputNotebook(wx.Notebook):
 	def save_all_editors(self, emr=None, rfe=None, aoe=None):
 
 		all_closed = True
-		for page_idx in range(self.GetPageCount()):
+		#for page_idx in range(self.GetPageCount()):
+		for page_idx in range((self.GetPageCount() - 1), 0, -1):
+			try: self.ChangeSelection(page_idx)
+			except: pass
 			page = self.GetPage(page_idx)
 			if page.save(emr = emr, rfe = rfe, aoe = aoe):
 				self.DeletePage(page_idx)
@@ -1218,6 +1249,8 @@ class cSoapNoteInputNotebook(wx.Notebook):
 		page = self.GetPage(page_idx)
 		return page.problem
 #============================================================
+from Gnumed.wxGladeWidgets import wxgSoapNoteExpandoEditAreaPnl
+
 class cSoapNoteExpandoEditAreaPnl(wxgSoapNoteExpandoEditAreaPnl.wxgSoapNoteExpandoEditAreaPnl):
 
 	def __init__(self, *args, **kwargs):
@@ -1388,6 +1421,38 @@ class cSoapLineTextCtrl(wxexpando.ExpandoTextCtrl):
 
 		self.__register_interests()
 	#------------------------------------------------
+	# fixup errors in platform expando.py
+	#------------------------------------------------
+	def _wrapLine(self, line, dc, width):
+
+		if (wx.MAJOR_VERSION > 1) and (wx.MINOR_VERSION > 8):
+			return super(cSoapLineTextCtrl, self)._wrapLine(line, dc, width)
+
+		# THIS FIX LIFTED FROM TRUNK IN SVN:
+		# Estimate where the control will wrap the lines and
+		# return the count of extra lines needed.
+		pte = dc.GetPartialTextExtents(line)
+		width -= wx.SystemSettings.GetMetric(wx.SYS_VSCROLL_X)
+		idx = 0
+		start = 0
+		count = 0
+		spc = -1
+		while idx < len(pte):
+		    if line[idx] == ' ':
+		        spc = idx
+		    if pte[idx] - start > width:
+		        # we've reached the max width, add a new line
+		        count += 1
+		        # did we see a space? if so restart the count at that pos
+		        if spc != -1:
+		            idx = spc + 1
+		            spc = -1
+		        if idx < len(pte):
+		            start = pte[idx]
+		    else:
+		        idx += 1
+		return count
+	#------------------------------------------------
 	# event handling
 	#------------------------------------------------
 	def __register_interests(self):
@@ -1490,10 +1555,448 @@ class cSoapLineTextCtrl(wxexpando.ExpandoTextCtrl):
 
 		return
 #============================================================
+# visual progress notes
+#============================================================
+visual_progress_note_document_type = u'visual progress note'
+
+#============================================================
+def configure_visual_progress_note_editor():
+
+	def is_valid(value):
+
+		if value is None:
+			gmDispatcher.send (
+				signal = 'statustext',
+				msg = _('You need to actually set an editor.'),
+				beep = True
+			)
+			return False, value
+
+		if value.strip() == u'':
+			gmDispatcher.send (
+				signal = 'statustext',
+				msg = _('You need to actually set an editor.'),
+				beep = True
+			)
+			return False, value
+
+		found, binary = gmShellAPI.detect_external_binary(value)
+		if not found:
+			gmDispatcher.send (
+				signal = 'statustext',
+				msg = _('The command [%s] is not found.') % value,
+				beep = True
+			)
+			return True, value
+
+		return True, binary
+	#------------------------------------------
+	gmCfgWidgets.configure_string_option (
+		message = _(
+			'Enter the shell command with which to start\n'
+			'the image editor for visual progress notes.\n'
+			'\n'
+			'Any "%(img)s" included with the arguments\n'
+			'will be replaced by the file name of the\n'
+			'note template.'
+		),
+		option = u'external.tools.visual_soap_editor_cmd',
+		bias = 'user',
+		default_value = None,
+		validator = is_valid
+	)
+#============================================================
+def edit_visual_progress_note(filename=None, episode=None, discard_unmodified=False, doc_part=None):
+	"""This assumes <filename> contains an image which can be handled by the configured image editor."""
+
+	if doc_part is not None:
+		filename = doc_part.export_to_file()
+		if filename is None:
+			gmDispatcher.send(signal = u'statustext', msg = _('Cannot export visual progress note to file.'))
+			return None
+
+	dbcfg = gmCfg.cCfgSQL()
+	cmd = dbcfg.get2 (
+		option = u'external.tools.visual_soap_editor_cmd',
+		workplace = gmSurgery.gmCurrentPractice().active_workplace,
+		bias = 'user'
+	)
+
+	if cmd is None:
+		gmDispatcher.send(signal = u'statustext', msg = _('Editor for visual progress note not configured.'), beep = False)
+		cmd = configure_visual_progress_note_editor()
+		if cmd is None:
+			gmDispatcher.send(signal = u'statustext', msg = _('Editor for visual progress note not configured.'), beep = True)
+			return None
+
+	if u'%(img)s' in cmd:
+		cmd % {u'img': filename}
+	else:
+		cmd = u'%s %s' % (cmd, filename)
+
+	if discard_unmodified:
+		original_stat = os.stat(filename)
+		original_md5 = gmTools.file2md5(filename)
+
+	success = gmShellAPI.run_command_in_shell(cmd, blocking = True)
+	if not success:
+		gmGuiHelpers.gm_show_error (
+			_(
+				'There was a problem with running the editor\n'
+				'for visual progress notes.\n'
+				'\n'
+				' [%s]\n'
+				'\n'
+			) % cmd,
+			_('Editing visual progress note')
+		)
+		return None
+
+	try:
+		open(filename, 'r').close()
+	except StandardError:
+		_log.exception('problem accessing visual progress note file [%s]', filename)
+		gmGuiHelpers.gm_show_error (
+			_(
+				'There was a problem reading the visual\n'
+				'progress note from the file:\n'
+				'\n'
+				' [%s]\n'
+				'\n'
+			) % filename,
+			_('Saving visual progress note')
+		)
+		return None
+
+	if discard_unmodified:
+		modified_stat = os.stat(filename)
+		# same size ?
+		if original_stat.st_size == modified_stat.st_size:
+			modified_md5 = gmTools.file2md5(filename)
+			# same hash ?
+			if original_md5 == modified_md5:
+				_log.debug('visual progress note (template) not modified')
+				# ask user to decide
+				msg = _(
+					u'This visual progress note was created from a\n'
+					u'template in the database rather than from a file\n'
+					u'but the image was not modified at all.\n'
+					u'\n'
+					u'Do you want to still save the unmodified\n'
+					u'image as a visual progress note into the\n'
+					u'EMR of the patient ?'
+				)
+				save_unmodified = gmGuiHelpers.gm_show_question (
+					msg,
+					_('Saving visual progress note')
+				)
+				if not save_unmodified:
+					_log.debug('user discarded unmodified note')
+					return
+
+	if doc_part is not None:
+		doc_part.update_data_from_file(fname = filename)
+		doc_part.set_reviewed(technically_abnormal = False, clinically_relevant = True)
+		return None
+
+	if not isinstance(episode, gmEMRStructItems.cEpisode):
+		pat = gmPerson.gmCurrentPatient()
+		emr = pat.get_emr()
+		episode = emr.add_episode(episode_name = episode.strip(), is_open = False)
+
+	doc = gmDocumentWidgets.save_file_as_new_document (
+		filename = filename,
+		document_type = visual_progress_note_document_type,
+		episode = episode,
+		unlock_patient = True
+	)
+	doc.set_reviewed(technically_abnormal = False, clinically_relevant = True)
+
+	return doc
+#============================================================
+class cVisualSoapTemplatePhraseWheel(gmPhraseWheel.cPhraseWheel):
+	"""Phrasewheel to allow selection of visual SOAP template."""
+
+	def __init__(self, *args, **kwargs):
+
+		gmPhraseWheel.cPhraseWheel.__init__ (self, *args, **kwargs)
+
+		query = u"""
+SELECT
+	pk,
+	name_short
+FROM
+	ref.paperwork_templates
+WHERE
+	fk_template_type = (SELECT pk FROM ref.form_types WHERE name = '%s') AND (
+		name_long %%(fragment_condition)s
+			OR
+		name_short %%(fragment_condition)s
+	)
+ORDER BY name_short
+LIMIT 15
+"""	% visual_progress_note_document_type
+
+		mp = gmMatchProvider.cMatchProvider_SQL2(queries = [query])
+		mp.setThresholds(2, 3, 5)
+
+		self.matcher = mp
+		self.selection_only = True
+	#--------------------------------------------------------
+	def _data2instance(self):
+		if self.data is None:
+			return None
+
+		return gmForms.cFormTemplate(aPK_obj = self.data)
+#============================================================
+from Gnumed.wxGladeWidgets import wxgVisualSoapPnl
+
+class cVisualSoapPnl(wxgVisualSoapPnl.wxgVisualSoapPnl):
+
+	def __init__(self, *args, **kwargs):
+
+		wxgVisualSoapPnl.wxgVisualSoapPnl.__init__(self, *args, **kwargs)
+
+		# dummy episode to hold images
+		self.default_episode_name = _('visual progress notes')
+	#--------------------------------------------------------
+	# external API
+	#--------------------------------------------------------
+	def clear(self):
+		self._PRW_template.SetText(value = u'', data = None)
+		self._LCTRL_visual_soaps.set_columns([_('Sketches')])
+		self._LCTRL_visual_soaps.set_string_items()
+
+		self.show_image_and_metadata()
+	#--------------------------------------------------------
+	def refresh(self, patient=None, encounter=None):
+
+		self.clear()
+
+		if patient is None:
+			patient = gmPerson.gmCurrentPatient()
+
+		if not patient.connected:
+			return
+
+		emr = patient.get_emr()
+		if encounter is None:
+			encounter = emr.active_encounter
+
+		folder = patient.get_document_folder()
+		soap_docs = folder.get_documents (
+			doc_type = visual_progress_note_document_type,
+			encounter = encounter['pk_encounter']
+		)
+
+		if len(soap_docs) == 0:
+			self._BTN_delete.Enable(False)
+			return
+
+		self._LCTRL_visual_soaps.set_string_items ([
+			u'%s%s%s' % (
+				gmTools.coalesce(sd['comment'], u'', u'%s\n'),
+				gmTools.coalesce(sd['ext_ref'], u'', u'%s\n'),
+				sd['episode']
+			) for sd in soap_docs
+		])
+		self._LCTRL_visual_soaps.set_data(soap_docs)
+
+		self._BTN_delete.Enable(True)
+	#--------------------------------------------------------
+	def show_image_and_metadata(self, doc=None):
+
+		if doc is None:
+			self._IMG_soap.SetBitmap(wx.NullBitmap)
+			self._PRW_episode.SetText()
+			#self._PRW_comment.SetText(value = u'', data = None)
+			self._PRW_comment.SetValue(u'')
+			return
+
+		parts = doc.parts
+		if len(parts) == 0:
+			gmDispatcher.send(signal = u'statustext', msg = _('No images in visual progress note.'))
+			return
+
+		fname = parts[0].export_to_file()
+		if fname is None:
+			gmDispatcher.send(signal = u'statustext', msg = _('Cannot export visual progress note to file.'))
+			return
+
+		img_data = None
+		rescaled_width = 300
+		try:
+			img_data = wx.Image(fname, wx.BITMAP_TYPE_ANY)
+			current_width = img_data.GetWidth()
+			current_height = img_data.GetHeight()
+			rescaled_height = (rescaled_width * current_height) / current_width
+			img_data.Rescale(rescaled_width, rescaled_height, quality = wx.IMAGE_QUALITY_HIGH)		# w, h
+			bmp_data = wx.BitmapFromImage(img_data)
+		except:
+			_log.exception('cannot load visual progress note from [%s]', fname)
+			gmDispatcher.send(signal = u'statustext', msg = _('Cannot load visual progress note from [%s].') % fname)
+			del img_data
+			return
+
+		del img_data
+		self._IMG_soap.SetBitmap(bmp_data)
+
+		self._PRW_episode.SetText(value = doc['episode'], data = doc['pk_episode'])
+		if doc['comment'] is not None:
+			self._PRW_comment.SetValue(doc['comment'].strip())
+	#--------------------------------------------------------
+	# event handlers
+	#--------------------------------------------------------
+	def _on_visual_soap_selected(self, event):
+
+		doc = self._LCTRL_visual_soaps.get_selected_item_data(only_one = True)
+		self.show_image_and_metadata(doc = doc)
+		if doc is None:
+			return
+
+		self._BTN_delete.Enable(True)
+	#--------------------------------------------------------
+	def _on_visual_soap_deselected(self, event):
+		self._BTN_delete.Enable(False)
+	#--------------------------------------------------------
+	def _on_visual_soap_activated(self, event):
+
+		doc = self._LCTRL_visual_soaps.get_selected_item_data(only_one = True)
+		if doc is None:
+			self.show_image_and_metadata()
+			return
+
+		parts = doc.parts
+		if len(parts) == 0:
+			gmDispatcher.send(signal = u'statustext', msg = _('No images in visual progress note.'))
+			return
+
+		edit_visual_progress_note(doc_part = parts[0], discard_unmodified = True)
+		self.show_image_and_metadata(doc = doc)
+
+		self._BTN_delete.Enable(True)
+	#--------------------------------------------------------
+	def _on_from_template_button_pressed(self, event):
+
+		template = self._PRW_template.GetData(as_instance = True)
+		if template is None:
+			return
+
+		filename = template.export_to_file()
+		if filename is None:
+			gmDispatcher.send(signal = u'statustext', msg = _('Cannot export visual progress note template for [%s].') % template['name_long'])
+			return
+
+		episode = self._PRW_episode.GetData(as_instance = True)
+		if episode is None:
+			episode = self._PRW_episode.GetValue().strip()
+			if episode == u'':
+				episode = self.default_episode_name
+
+		# do not store note if not modified -- change if users complain
+		doc = edit_visual_progress_note(filename = filename, episode = episode, discard_unmodified = True)
+		if doc is None:
+			return
+
+		if self._PRW_comment.GetValue().strip() == u'':
+			doc['comment'] = template['instance_type']
+		else:
+			doc['comment'] = self._PRW_comment.GetValue().strip()
+
+		doc.save()
+		self.show_image_and_metadata(doc = doc)
+	#--------------------------------------------------------
+	def _on_from_file_button_pressed(self, event):
+
+		dlg = wx.FileDialog (
+			parent = self,
+			message = _('Choose a visual progress note template file'),
+			defaultDir = os.path.expanduser('~'),
+			defaultFile = '',
+			#wildcard = "%s (*)|*|%s (*.*)|*.*" % (_('all files'), _('all files (Win)')),
+			style = wx.OPEN | wx.HIDE_READONLY | wx.FILE_MUST_EXIST
+		)
+		result = dlg.ShowModal()
+		if result == wx.ID_CANCEL:
+			dlg.Destroy()
+			return
+
+		full_filename = dlg.GetPath()
+		dlg.Hide()
+		dlg.Destroy()
+
+		# create a copy of the picked file -- don't modify the original
+		ext = os.path.splitext(full_filename)[1]
+		tmp_name = gmTools.get_unique_filename(suffix = ext)
+		_log.debug('visual progress note from file: [%s] -> [%s]', full_filename, tmp_name)
+		shutil.copy2(full_filename, tmp_name)
+
+		episode = self._PRW_episode.GetData(as_instance = True)
+		if episode is None:
+			episode = self._PRW_episode.GetValue().strip()
+			if episode == u'':
+				episode = self.default_episode_name
+
+		# always store note even if unmodified as we
+		# may simply want to store a clinical photograph
+		doc = edit_visual_progress_note(filename = tmp_name, episode = episode, discard_unmodified = False)
+		if self._PRW_comment.GetValue().strip() == u'':
+			# use filename as default comment (w/o extension)
+			doc['comment'] = os.path.splitext(os.path.split(full_filename)[1])[0]
+		else:
+			doc['comment'] = self._PRW_comment.GetValue().strip()
+		doc.save()
+		self.show_image_and_metadata(doc = doc)
+
+		try:
+			os.remove(tmp_name)
+		except StandardError:
+			_log.exception('cannot remove [%s]', tmp_name)
+
+		remove_original = gmGuiHelpers.gm_show_question (
+			_(
+				'Do you want to delete the original file\n'
+				'\n'
+				' [%s]\n'
+				'\n'
+				'from your computer ?'
+			) % full_filename,
+			_('Saving visual progress note ...')
+		)
+		if remove_original:
+			try:
+				os.remove(full_filename)
+			except StandardError:
+				_log.exception('cannot remove [%s]', full_filename)
+	#--------------------------------------------------------
+	def _on_delete_button_pressed(self, event):
+
+		doc = self._LCTRL_visual_soaps.get_selected_item_data(only_one = True)
+		if doc is None:
+			self.show_image_and_metadata()
+			return
+
+		delete_it = gmGuiHelpers.gm_show_question (
+			aMessage = _('Are you sure you want to delete the visual progress note ?'),
+			aTitle = _('Deleting visual progress note')
+		)
+		if delete_it is True:
+			gmDocuments.delete_document (
+				document_id = doc['pk_doc'],
+				encounter_id = doc['pk_encounter']
+			)
+		self.show_image_and_metadata()
+#============================================================
 # main
 #------------------------------------------------------------
 if __name__ == '__main__':
 
+	if len(sys.argv) < 2:
+		sys.exit()
+
+	if sys.argv[1] != 'test':
+		sys.exit()
+
 	gmI18N.activate_locale()
 	gmI18N.install_domain(domain = 'gnumed')
 
@@ -1527,164 +2030,9 @@ if __name__ == '__main__':
 		soap_input._schedule_data_reget()
 		application.MainLoop()
 	#----------------------------------------
-	if (len(sys.argv) > 1) and (sys.argv[1] == 'test'):
-		#test_select_narrative_from_episodes()
-		test_cSoapNoteExpandoEditAreaPnl()
-		#test_cSoapPluginPnl()
+	#test_select_narrative_from_episodes()
+	test_cSoapNoteExpandoEditAreaPnl()
+	#test_cSoapPluginPnl()
 
 #============================================================
-# $Log: gmNarrativeWidgets.py,v $
-# Revision 1.45  2010-01-11 19:51:09  ncq
-# - cleanup
-# - warn-on-unsaved-soap and use in syn pre-selection callback
-#
-# Revision 1.44  2009/11/28 18:32:50  ncq
-# - finalize showing potential problems in problem list, too, and adjust box label
-#
-# Revision 1.43  2009/11/24 21:03:41  ncq
-# - display problems based on checkbox selection
-# - set recent notes label based on problem selection
-#
-# Revision 1.42  2009/11/15 01:10:09  ncq
-# - enhance move-progress-notes-to-another-encounter
-# - use enhanced new encounter start
-#
-# Revision 1.41  2009/11/13 21:08:24  ncq
-# - enable cross-EMR narrative search to activate matching
-#   patient from result list
-#
-# Revision 1.40  2009/11/08 20:49:49  ncq
-# - implement search across all EMRs
-#
-# Revision 1.39  2009/09/13 18:45:25  ncq
-# - no more get-active-encounter()
-#
-# Revision 1.38  2009/09/01 22:36:59  ncq
-# - wx-CallAfter on start-new-encounter
-#
-# Revision 1.37  2009/07/23 16:41:13  ncq
-# - cleanup
-#
-# Revision 1.36  2009/07/02 20:55:48  ncq
-# - properly honor allow-same-problem on non-new editors only
-#
-# Revision 1.35  2009/07/01 17:09:06  ncq
-# - refresh fields explicitly when active encounter is switched
-#
-# Revision 1.34  2009/06/29 15:09:45  ncq
-# - inform user when nothing is found during search
-# - refresh recent-notes on problem single-click selection
-#   but NOT anymore on editor changes
-#
-# Revision 1.33  2009/06/22 09:28:20  ncq
-# - improved wording as per list
-#
-# Revision 1.32  2009/06/20 22:39:27  ncq
-# - improved wording as per list discussion
-#
-# Revision 1.31  2009/06/11 12:37:25  ncq
-# - much simplified initial setup of list ctrls
-#
-# Revision 1.30  2009/06/04 16:33:13  ncq
-# - adjust to dob-less person
-# - use set-active-patient from pat-search-widgets
-#
-# Revision 1.29  2009/05/13 13:12:41  ncq
-# - cleanup
-#
-# Revision 1.28  2009/05/13 12:22:05  ncq
-# - move_progress_notes_to_another_encounter
-#
-# Revision 1.27  2009/04/16 12:51:02  ncq
-# - edit_* -> manage_progress_notes as it can delete now, too,
-#   after being converted to using get_choices_from_list
-#
-# Revision 1.26  2009/04/13 10:56:21  ncq
-# - use same_payload on encounter to detect changes
-# - detect when current encounter is switched, not just modified
-#
-# Revision 1.25  2009/03/10 14:23:56  ncq
-# - comment
-#
-# Revision 1.24  2009/03/02 18:57:52  ncq
-# - make expando soap editor scroll to cursor when needed
-#
-# Revision 1.23  2009/02/24 13:22:06  ncq
-# - fix saving edited progress notes
-#
-# Revision 1.22  2009/02/17 08:07:37  ncq
-# - support explicit macro expansion
-#
-# Revision 1.21  2009/01/21 22:37:14  ncq
-# - do not fail save_all_editors() where not appropriate
-#
-# Revision 1.20  2009/01/21 18:53:57  ncq
-# - fix save_all_editors and call it with proper args
-#
-# Revision 1.19  2009/01/03 17:29:01  ncq
-# - listen on new current_encounter_modified
-# - detecting encounter field changes at exit/patient doesn't properly work
-# - refresh recent notes where needed
-#
-# Revision 1.18  2009/01/02 11:41:16  ncq
-# - improved event handling
-#
-# Revision 1.17  2008/12/27 15:50:41  ncq
-# - almost finish implementing soap saving
-#
-# Revision 1.16  2008/12/26 22:35:44  ncq
-# - edit_progress_notes
-# - implement most of new soap plugin functionality
-#
-# Revision 1.15  2008/11/24 11:10:29  ncq
-# - cleanup
-#
-# Revision 1.14  2008/11/23 12:47:02  ncq
-# - preset splitter ratios and gravity
-# - cleanup
-# - reorder recent notes with most recent on bottom as per list
-#
-# Revision 1.13  2008/11/20 20:35:50  ncq
-# - new soap plugin widgets
-#
-# Revision 1.12  2008/10/26 01:21:52  ncq
-# - factor out searching EMR for narrative
-#
-# Revision 1.11  2008/10/22 12:21:57  ncq
-# - use %x in strftime where appropriate
-#
-# Revision 1.10  2008/10/12 16:26:20  ncq
-# - consultation -> encounter
-#
-# Revision 1.9  2008/09/02 19:01:12  ncq
-# - adjust to clin health_issue fk_patient drop and related changes
-#
-# Revision 1.8  2008/07/28 15:46:05  ncq
-# - export_narrative_for_medistar_import
-#
-# Revision 1.7  2008/03/05 22:30:14  ncq
-# - new style logging
-#
-# Revision 1.6  2007/12/03 20:45:28  ncq
-# - improved docs
-#
-# Revision 1.5  2007/09/10 12:36:02  ncq
-# - improved wording in narrative selector at SOAP level
-#
-# Revision 1.4  2007/09/09 19:21:04  ncq
-# - get top level wx.App window if parent is None
-# - support filtering by soap_cats
-#
-# Revision 1.3  2007/09/07 22:45:58  ncq
-# - much improved select_narrative_from_episodes()
-#
-# Revision 1.2  2007/09/07 10:59:17  ncq
-# - greatly improve select_narrative_by_episodes
-#   - remember selections
-#   - properly levelled looping
-# - fix test suite
-#
-# Revision 1.1  2007/08/29 22:06:15  ncq
-# - factored out narrative widgets
-#
-#
+
