@@ -7,9 +7,7 @@ interface of Richard Terry's Visual Basic client
 This is based on seminal work by Ian Haywood <ihaywood@gnu.org>
 """
 ############################################################################
-# $Source: /home/ncq/Projekte/cvs2git/vcs-mirror/gnumed/gnumed/client/wxpython/gmPhraseWheel.py,v $
-# $Id: gmPhraseWheel.py,v 1.135 2009-11-15 01:10:53 ncq Exp $
-__version__ = "$Revision: 1.135 $"
+__version__ = "$Revision: 1.136 $"
 __author__  = "K.Hilbert <Karsten.Hilbert@gmx.net>, I.Haywood, S.J.Tan <sjtan@bigpond.com>"
 __license__ = "GPL"
 
@@ -332,9 +330,19 @@ class cPhraseWheel(wx.TextCtrl):
 		self.display_as_valid(valid = True)
 		return True
 	#---------------------------------------------------------
-	def GetData(self):
+	def GetData(self, can_create=False, as_instance=False):
 		"""Retrieve the data associated with the displayed string.
+
+			_create_data() must set self.data if possible
 		"""
+		if self.data is None:
+			if can_create:
+				self._create_data()
+
+		if self.data is not None:
+			if as_instance:
+				return self._data2instance()
+
 		return self.data
 	#---------------------------------------------------------
 	def SetText(self, value=u'', data=None, suppress_smarts=False):
@@ -344,6 +352,8 @@ class cPhraseWheel(wx.TextCtrl):
 		if data is not None:
 			self.suppress_text_update_smarts = True
 			self.data = data
+		if value is None:
+			value = u''
 		wx.TextCtrl.SetValue(self, value)
 		self.display_as_valid(valid = True)
 
@@ -661,6 +671,9 @@ class cPhraseWheel(wx.TextCtrl):
 	#--------------------------------------------------------
 	# internal helpers: logic
 	#--------------------------------------------------------
+	def _create_data(self):
+		raise NotImplementedError('[%s]: cannot create data object' % self.__class__.__name__)
+	#--------------------------------------------------------
 	def __char_is_allowed(self, char=None):
 		# if undefined accept all chars
 		if self.accepted_chars is None:
@@ -948,6 +961,13 @@ class cPhraseWheel(wx.TextCtrl):
 # MAIN
 #--------------------------------------------------------
 if __name__ == '__main__':
+
+	if len(sys.argv) < 2:
+		sys.exit()
+
+	if sys.argv[1] != u'test':
+		sys.exit()
+
 	from Gnumed.pycommon import gmI18N
 	gmI18N.activate_locale()
 	gmI18N.install_domain(domain='gnumed')
@@ -1068,493 +1088,3 @@ if __name__ == '__main__':
 #	test_prw_patients()
 
 #==================================================
-# $Log: gmPhraseWheel.py,v $
-# Revision 1.135  2009-11-15 01:10:53  ncq
-# - cleanup
-#
-# Revision 1.134  2009/07/23 16:41:42  ncq
-# - support custom error msg on final regex mismatch
-#
-# Revision 1.133  2009/04/24 12:06:01  ncq
-# - fix application of final regex: if it was compiled with LOCALE/UNICODE
-#   one *cannot* apply those flags again on matching !
-#
-# Revision 1.132  2009/04/19 22:27:36  ncq
-# - enlarge edit font by 1 point only
-#
-# Revision 1.131  2009/04/03 09:52:10  ncq
-# - add explicit shutdown for timers
-# - self-handle timers
-# - a bit of cleanup
-#
-# Revision 1.130  2009/03/31 15:08:09  ncq
-# - removed gmTimer dependancy
-#
-# Revision 1.129  2009/03/31 14:38:13  ncq
-# - rip out gmDispatcher and use wx.lib.pubsub
-#
-# Revision 1.128  2009/03/01 18:18:50  ncq
-# - factor out default phrase separators/spelling word separators
-#
-# Revision 1.127  2009/02/24 10:16:48  ncq
-# - don't hiccup when spell checker hiccups, simply disable it
-#
-# Revision 1.126  2009/02/04 21:47:54  ncq
-# - cleanup
-#
-# Revision 1.125  2008/10/12 16:32:40  ncq
-# - make more robust when getting data of selected item
-#
-# Revision 1.124  2008/08/15 15:57:37  ncq
-# - enchant doesn't like spellchecking '' anymore
-#
-# Revision 1.123  2008/07/13 16:14:00  ncq
-# - outside code uses color_prw_valid, so leave it and add a comment
-#
-# Revision 1.122  2008/07/07 11:39:21  ncq
-# - separate fake_popup from needs_relative_pos flag
-#
-# Revision 1.121  2008/06/26 17:03:53  ncq
-# - use a wxMiniFrame instead of a wx.Window when emulating wx.PopupWindow
-# - adjust for some extra space needed by the wx.MiniFrame
-#
-# Revision 1.120  2008/06/18 15:48:21  ncq
-# - support valid/invalid coloring via display_as_valid
-# - cleanup init flow
-#
-# Revision 1.119  2008/06/15 20:40:43  ncq
-# - adjust test suite to match provider properties
-#
-# Revision 1.118  2008/06/09 15:36:39  ncq
-# - increase font size by 2 points when editing
-#
-# Revision 1.117  2008/05/14 13:46:37  ncq
-# - better logging
-#
-# Revision 1.116  2008/05/13 14:15:16  ncq
-# - TAB = select-single-match only when selection_only True
-# - improve wxPopupWindow emulation
-#
-# Revision 1.115  2008/05/07 15:21:44  ncq
-# - support suppress smarts argument to SetText
-#
-# Revision 1.114  2008/04/26 16:29:15  ncq
-# - missing if
-#
-# Revision 1.113  2008/04/26 10:06:37  ncq
-# - on MacOSX use relative position for popup window
-#
-# Revision 1.112  2008/04/26 09:30:28  ncq
-# - instrument phrasewheel to exhibit Mac problem
-#   with dropdown placement
-#
-# Revision 1.111  2008/01/30 14:03:42  ncq
-# - use signal names directly
-# - switch to std lib logging
-#
-# Revision 1.110  2007/10/29 11:30:21  ncq
-# - rephrase TODOs
-#
-# Revision 1.109  2007/09/02 20:56:30  ncq
-# - cleanup
-#
-# Revision 1.108  2007/08/12 00:12:41  ncq
-# - no more gmSignals.py
-#
-# Revision 1.107  2007/07/10 20:27:27  ncq
-# - install_domain() arg consolidation
-#
-# Revision 1.106  2007/07/03 16:03:04  ncq
-# - cleanup
-# - compile final_regex_error_msg just before using it
-#   since self.final_regex can have changed
-#
-# Revision 1.105  2007/05/14 14:43:11  ncq
-# - allow TAB to select item from picklist if only one match available
-#
-# Revision 1.104  2007/05/14 13:11:25  ncq
-# - use statustext() signal
-#
-# Revision 1.103  2007/04/19 13:14:30  ncq
-# - don't fail input if enchant/aspell installed but no dict available ...
-#
-# Revision 1.102  2007/04/02 15:16:55  ncq
-# - make spell checker act on last word of phrase only
-# - to that end add property speller_word_separators, a
-#   regex which defaults to standard word boundaries + digits + _
-#
-# Revision 1.101  2007/04/02 14:31:35  ncq
-# - cleanup
-#
-# Revision 1.100  2007/04/01 16:33:47  ncq
-# - try another parent for the MacOSX popup window
-#
-# Revision 1.99  2007/03/31 20:09:06  ncq
-# - make enchant optional
-#
-# Revision 1.98  2007/03/27 10:29:49  ncq
-# - better placement for default word list
-#
-# Revision 1.97  2007/03/27 09:59:26  ncq
-# - enable_default_spellchecker()
-#
-# Revision 1.96  2007/02/16 10:22:09  ncq
-# - _calc_display_string -> _picklist_selection2display_string to better reflect its use
-#
-# Revision 1.95  2007/02/06 13:45:39  ncq
-# - much improved docs
-# - remove aDelay from __init__ and make it a class variable
-# - thereby we can now dynamically adjust it at runtime :-)
-# - add patient searcher phrasewheel example
-#
-# Revision 1.94  2007/02/05 12:11:17  ncq
-# - put GPL into __license__
-# - code and layout cleanup
-# - remove dependancy on gmLog
-# - cleanup __init__ interface:
-# 	- remove selection_only
-# 	- remove aMatchProvider
-# 	- set both directly on instance members now
-# - implement spell checking plus test case for it
-# - implement configurable error messages
-#
-# Revision 1.93  2007/02/04 18:50:12  ncq
-# - capitalisation_mode is now instance variable
-#
-# Revision 1.92  2007/02/04 16:04:03  ncq
-# - reduce imports
-# - add accepted_chars constants
-# - enhance phrasewheel:
-#   - better credits
-#   - cleaner __init__ signature
-#   - user properties
-#   - code layout/naming cleanup
-#   - no more snap_to_first_match for now
-#   - add capitalisation mode
-#   - add accepted chars checking
-#   - add final regex matching
-#   - allow suppressing recursive _on_text_update()
-#   - always use time, even in slave mode
-#   - lots of logic consolidation
-#   - add SetText() and favour it over SetValue()
-#
-# Revision 1.91  2007/01/20 22:52:27  ncq
-# - .KeyCode -> GetKeyCode()
-#
-# Revision 1.90  2007/01/18 22:07:52  ncq
-# - (Get)KeyCode() -> KeyCode so 2.8 can do
-#
-# Revision 1.89  2007/01/06 23:44:19  ncq
-# - explicitely unset selection on lose focus
-#
-# Revision 1.88  2006/11/28 20:51:13  ncq
-# - a missing self
-# - remove some prints
-#
-# Revision 1.87  2006/11/27 23:08:36  ncq
-# - add snap_to_first_match
-# - add on_modified callbacks
-# - set background in lose_focus in some cases
-# - improve test suite
-#
-# Revision 1.86  2006/11/27 12:42:31  ncq
-# - somewhat improved dropdown picklist on Mac, not properly positioned yet
-#
-# Revision 1.85  2006/11/26 21:42:47  ncq
-# - don't use wx.ScrolledWindow or we suffer double-scrollers
-#
-# Revision 1.84  2006/11/26 20:58:20  ncq
-# - try working around lacking wx.PopupWindow
-#
-# Revision 1.83  2006/11/26 14:51:19  ncq
-# - cleanup/improve test suite so we can get MacOSX nailed (down)
-#
-# Revision 1.82  2006/11/26 14:09:59  ncq
-# - fix sys.path when running standalone for test suite
-# - fix test suite
-#
-# Revision 1.81  2006/11/24 09:58:39  ncq
-# - cleanup
-# - make it really work when matcher is None
-#
-# Revision 1.80  2006/11/19 11:16:02  ncq
-# - remove self._input_was_selected
-#
-# Revision 1.79  2006/11/06 12:54:00  ncq
-# - we don't actually need self._input_was_selected thanks to self.data
-#
-# Revision 1.78  2006/11/05 16:10:11  ncq
-# - cleanup
-# - now really handle context
-# - add unset_context()
-# - stop timer in __init__()
-# - start timer in _on_set_focus()
-# - some u''-ification
-#
-# Revision 1.77  2006/10/25 07:24:51  ncq
-# - gmPG -> gmPG2
-# - match provider _SQL deprecated
-#
-# Revision 1.76  2006/07/19 20:29:50  ncq
-# - import cleanup
-#
-# Revision 1.75  2006/07/04 14:15:17  ncq
-# - lots of cleanup
-# - make dropdown list scroll !  :-)
-# - add customized list control
-# - don't make dropdown go below screen height
-#
-# Revision 1.74  2006/07/01 15:14:26  ncq
-# - lots of cleanup
-# - simple border around list dropdown
-# - remove on_resize handling
-# - remove setdependant()
-# - handle down-arrow to drop down list
-#
-# Revision 1.73  2006/07/01 13:14:50  ncq
-# - cleanup as gleaned from TextCtrlAutoComplete
-#
-# Revision 1.72  2006/06/28 22:16:08  ncq
-# - add SetData() -- which only works if data can be found in the match space
-#
-# Revision 1.71  2006/06/18 13:47:29  ncq
-# - set self.input_was_selected=True if SetValue() does have data with it
-#
-# Revision 1.70  2006/06/05 21:36:40  ncq
-# - cleanup
-#
-# Revision 1.69  2006/06/02 09:59:03  ncq
-# - must invalidate associated data object *as soon as*
-#   the text in the control changes
-#
-# Revision 1.68  2006/05/31 10:28:27  ncq
-# - cleanup
-# - deprecation warning for <id_callback> argument
-#
-# Revision 1.67  2006/05/25 22:24:20  ncq
-# - self.__input_was_selected -> self._input_was_selected
-#   because subclasses need access to it
-#
-# Revision 1.66  2006/05/24 09:47:34  ncq
-# - remove superfluous self._is_modified, use MarkDirty() instead
-# - cleanup SetValue()
-# - client data in picklist better be object, not string
-# - add _calc_display_string() for better reuse in subclasses
-# - fix "pick list windows too small if one match" at the cost of extra
-#   empty row when no horizontal scrollbar needed ...
-#
-# Revision 1.65  2006/05/20 18:54:15  ncq
-# - cleanup
-#
-# Revision 1.64  2006/05/01 18:49:49  ncq
-# - add_callback_on_set_focus()
-#
-# Revision 1.63  2005/10/09 08:15:21  ihaywood
-# SetValue () has optional second parameter to set data.
-#
-# Revision 1.62  2005/10/09 02:19:40  ihaywood
-# the address widget now has the appropriate widget order and behaviour for australia
-# when os.environ["LANG"] == 'en_AU' (is their a more graceful way of doing this?)
-#
-# Remember our postcodes work very differently.
-#
-# Revision 1.61  2005/10/04 00:04:45  sjtan
-# convert to wx.; catch some transitional errors temporarily
-#
-# Revision 1.60  2005/09/28 21:27:30  ncq
-# - a lot of wx2.6-ification
-#
-# Revision 1.59  2005/09/28 15:57:48  ncq
-# - a whole bunch of wx.Foo -> wx.Foo
-#
-# Revision 1.58  2005/09/26 18:01:51  ncq
-# - use proper way to import wx26 vs wx2.4
-# - note: THIS WILL BREAK RUNNING THE CLIENT IN SOME PLACES
-# - time for fixup
-#
-# Revision 1.57  2005/08/14 15:37:36  ncq
-# - cleanup
-#
-# Revision 1.56  2005/07/24 11:35:59  ncq
-# - use robustified gmTimer.Start() interface
-#
-# Revision 1.55  2005/07/23 21:55:40  shilbert
-# *** empty log message ***
-#
-# Revision 1.54  2005/07/23 21:10:58  ncq
-# - explicitely use milliseconds=-1 in timer.Start()
-#
-# Revision 1.53  2005/07/23 19:24:58  ncq
-# - debug timer start() on windows
-#
-# Revision 1.52  2005/07/04 11:20:59  ncq
-# - cleanup cruft
-# - on_set_focus() set value to first match if previously empty
-# - on_lose_focus() set value if selection_only and only one match and not yet selected
-#
-# Revision 1.51  2005/06/14 19:55:37  cfmoro
-# Set selection flag when setting value
-#
-# Revision 1.50  2005/06/07 10:18:23  ncq
-# - cleanup
-# - setContext -> set_context
-#
-# Revision 1.49  2005/06/01 23:09:02  ncq
-# - set default phrasewheel delay to 150ms
-#
-# Revision 1.48  2005/05/23 16:42:50  ncq
-# - when we SetValue(val) we need to only check those matches
-#   that actually *can* match, eg the output of getMatches(val)
-#
-# Revision 1.47  2005/05/22 23:09:13  cfmoro
-# Adjust the underlying data when setting the phrasewheel value
-#
-# Revision 1.46  2005/05/17 08:06:38  ncq
-# - support for callbacks on lost focus
-#
-# Revision 1.45  2005/05/14 15:06:48  ncq
-# - GetData()
-#
-# Revision 1.44  2005/05/05 06:31:06  ncq
-# - remove dead cWheelTimer code in favour of gmTimer.py
-# - add self._on_enter_callbacks and add_callback_on_enter()
-# - addCallback() -> add_callback_on_selection()
-#
-# Revision 1.43  2005/03/14 14:37:56  ncq
-# - only disable timer if slave mode is really active
-#
-# Revision 1.42  2004/12/27 16:23:39  ncq
-# - gmTimer callbacks take a cookie
-#
-# Revision 1.41  2004/12/23 16:21:21  ncq
-# - some cleanup
-#
-# Revision 1.40  2004/10/16 22:42:12  sjtan
-#
-# script for unitesting; guard for unit tests where unit uses gmPhraseWheel; fixup where version of wxPython doesn't allow
-# a child widget to be multiply inserted (gmDemographics) ; try block for later versions of wxWidgets that might fail
-# the Add (.. w,h, ... ) because expecting Add(.. (w,h) ...)
-#
-# Revision 1.39  2004/09/13 09:24:30  ncq
-# - don't start timers in slave_mode since cannot start from
-#   other than main thread, this is a dirty fix but will do for now
-#
-# Revision 1.38  2004/06/25 12:30:52  ncq
-# - use True/False
-#
-# Revision 1.37  2004/06/17 11:43:15  ihaywood
-# Some minor bugfixes.
-# My first experiments with wxGlade
-# changed gmPhraseWheel so the match provider can be added after instantiation
-# (as wxGlade can't do this itself)
-#
-# Revision 1.36  2004/05/02 22:53:53  ncq
-# - cleanup
-#
-# Revision 1.35  2004/05/01 10:27:47  shilbert
-# - self._picklist.Append() needs string or unicode object
-#
-# Revision 1.34  2004/03/05 11:22:35  ncq
-# - import from Gnumed.<pkg>
-#
-# Revision 1.33  2004/03/02 10:21:10  ihaywood
-# gmDemographics now supports comm channels, occupation,
-# country of birth and martial status
-#
-# Revision 1.32  2004/02/25 09:46:22  ncq
-# - import from pycommon now, not python-common
-#
-# Revision 1.31  2004/01/12 13:14:39  ncq
-# - remove dead code
-# - correctly calculate new pick list position: don't go to TOPLEVEL
-#   window but rather to immediate parent ...
-#
-# Revision 1.30  2004/01/06 10:06:02  ncq
-# - make SQL based phrase wheel test work again
-#
-# Revision 1.29  2003/11/19 23:42:00  ncq
-# - cleanup, comment out snap()
-#
-# Revision 1.28  2003/11/18 23:17:47  ncq
-# - cleanup, fixed variable names
-#
-# Revision 1.27  2003/11/17 10:56:38  sjtan
-#
-# synced and commiting.
-#
-# Revision 1.26  2003/11/09 14:28:30  ncq
-# - cleanup
-#
-# Revision 1.25  2003/11/09 02:24:42  ncq
-# - added Syans "input was selected from list" state flag to avoid unnecessary list
-#   drop downs
-# - variable name cleanup
-#
-# Revision 1.24  2003/11/07 20:48:04  ncq
-# - place comments where they belong
-#
-# Revision 1.23  2003/11/05 22:21:06  sjtan
-#
-# let's gmDateInput specify id_callback in constructor list.
-#
-# Revision 1.22  2003/11/04 10:35:23  ihaywood
-# match providers in gmDemographicRecord
-#
-# Revision 1.21  2003/11/04 01:40:27  ihaywood
-# match providers moved to python-common
-#
-# Revision 1.20  2003/10/26 11:27:10  ihaywood
-# gmPatient is now the "patient stub", all demographics stuff in gmDemographics.
-#
-# Ergregious breakages are fixed, but needs more work
-#
-# Revision 1.19  2003/10/09 15:45:16  ncq
-# - validate cookie column in score tables, too
-#
-# Revision 1.18  2003/10/07 22:20:50  ncq
-# - ported Syan's extra_sql_condition extension
-# - make SQL match provider aware of separate scoring tables
-#
-# Revision 1.17  2003/10/03 00:20:25  ncq
-# - handle case where matches = 1 and match = input -> don't show picklist
-#
-# Revision 1.16  2003/10/02 20:51:12  ncq
-# - add alt-XX shortcuts, move __* to _*
-#
-# Revision 1.15  2003/09/30 18:52:40  ncq
-# - factored out date input wheel
-#
-# Revision 1.14  2003/09/29 23:11:58  ncq
-# - add __explicit_offset() date expander
-#
-# Revision 1.13  2003/09/29 00:16:55  ncq
-# - added date match provider
-#
-# Revision 1.12  2003/09/21 10:55:04  ncq
-# - coalesce merge conflicts due to optional SQL phrase wheel testing
-#
-# Revision 1.11  2003/09/21 07:52:57  ihaywood
-# those bloody umlauts killed by python interpreter!
-#
-# Revision 1.10  2003/09/17 05:54:32  ihaywood
-# phrasewheel box size now approximate to length of search results
-#
-# Revision 1.8  2003/09/16 22:25:45  ncq
-# - cleanup
-# - added first draft of single-column-per-table SQL match provider
-# - added module test for SQL matcher
-#
-# Revision 1.7  2003/09/15 16:05:30  ncq
-# - allow several phrases to be typed in and only try to match
-#   the one the cursor is in at the moment
-#
-# Revision 1.6  2003/09/13 17:46:29  ncq
-# - pattern match word separators
-# - pattern match ignore characters as per Richard's suggestion
-# - start work on phrase separator pattern matching with extraction of
-#   relevant input part (where the cursor is at currently)
-#
-# Revision 1.5  2003/09/10 01:50:25  ncq
-# - cleanup
-#
-#
